/* ccontrol.cc
 * Authors: Daniel Karrels dan@karrels.com
	    Tomer Cohen    MrBean@toughguy.net
 */

#include	<new>
#include	<string>
#include	<vector>
#include	<iostream>
#include	<algorithm>
#include 	<fstream>
#include	<cstring>
#include	<csignal>

#include        <sys/types.h> 
#include        <sys/socket.h>
#include        <netinet/in.h>
#include        <netdb.h>
#include	<fcntl.h>

#include	"client.h"
#include	"iClient.h"
#include	"EConfig.h"
#include	"events.h"
#include	"StringTokenizer.h"
#include	"misc.h"
#include	"Network.h"
#include	"ELog.h"
#include        "ccUser.h"
#include	"libpq++.h"
#include	"ccontrol.h"
#include        "server.h"
#include 	"Constants.h"
#include	"commLevels.h"
#include	"ccFloodData.h"
#include	"ccUserData.h"
#include	"ccGate.h"
#include	"ip.h"

const char CControl_h_rcsId[] = __CCONTROL_H ;
const char CControl_cc_rcsId[] = "$Id: ccontrol.cc,v 1.141 2002/04/26 16:02:49 dan_karrels Exp $" ;

namespace gnuworld
{

using std::string ;
using std::vector ;
using std::cout ;
using std::endl ; 
using std::count ;

namespace uworld
{

using gnuworld::xServer;
using namespace std;

/*
 *  Exported function used by moduleLoader to gain an
 *  instance of this module.
 */

extern "C"
{
  xClient* _gnuwinit(const string& args)
  { 
    return new ccontrol( args );
  }

} 

unsigned int dbConnected = false;
 
ccontrol::ccontrol( const string& configFileName )
 : xClient( configFileName )
{

// Read the config file
EConfig conf( configFileName ) ;

sqlHost = conf.Find("sql_host" )->second;
sqlDb = conf.Find( "sql_db" )->second;
sqlPort = conf.Find( "sql_port" )->second;
sqlPass = conf.Require( "sql_pass" )->second;
sqlUser = conf.Require( "sql_user" )->second;

inBurst = true;
inRefresh = false;

string Query = "host=" + sqlHost + " dbname=" + sqlDb + " port=" + sqlPort;
if (strcasecmp(sqlUser,"''"))
	{
	Query += (" user=" + sqlUser);
	}

if (strcasecmp(sqlPass,"''"))
	{
	Query += (" password=" + sqlPass);
	}
	
elog << Query << "\n";
elog	<< "ccontrol::ccontrol> Attempting to connect to "
	<< sqlHost
	<< "; Database: "
	<< sqlDb
	<< endl;
 
SQLDb = new (std::nothrow) cmDatabase( Query.c_str() ) ;
assert( SQLDb != 0 ) ;

//-- Make sure we connected to the SQL database; if
// we didn't we exit entirely.
if (SQLDb->ConnectionBad ())
	{
	elog	<< "ccontrol::ccontrol> Unable to connect to SQL server."
		<< endl 
		<< "ccontrol::ccontrol> PostgreSQL error message: "
		<< SQLDb->ErrorMessage()
		<< endl ;

	::exit( 0 ) ;
	}
else
	{
	elog	<< "ccontrol::ccontrol> Connection established to SQL "
		<< endl ;
	}
dbConnected = true;

// operChanReason is the reason used when kicking non-opers from
// oper-only channels
operChanReason = conf.Find( "operchanreason" )->second ;

// operChanModes are the modes to set when setting up an oper-only
// channel
operChanModes = conf.Find( "operchanmodes" )->second ;

// gLength is the length of time (in seconds) for default glines
gLength = atoi( conf.Find( "glength" )->second.c_str() ) ;

// CCEmail is the email ccontrol will post the last com report under
CCEmail = conf.Require( "ccemail" )->second;

//AbuseMail is the mail that the lastcom report will be post to
AbuseMail = conf.Require( "abuse_mail" )->second;

//GLInterval is the inteval in which ccontrol will check for expired glines
ExpiredInterval = atoi( conf.Require( "Expired_interval" )->second.c_str() );

//Sendmail  is the full path of the sendmail program
Sendmail_Path = conf.Require("SendMail")->second;

//SendReport flag that tells ccontrol if the user want the report to be mailed
SendReport = atoi(conf.Require("mail_report")->second.c_str());

userMaxConnection = atoi(conf.Require("max_connection")->second.c_str());

maxGlineLen = atoi(conf.Require("max_GLen")->second.c_str());

maxThreads = atoi(conf.Require("max_threads")->second.c_str());

checkGates = atoi(conf.Require("check_gates")->second.c_str());

checkClones = atoi(conf.Require("check_clones")->second.c_str());

dbConnectionTimer = atoi(conf.Require("dbinterval")->second.c_str());

// Set up the oper channels
EConfig::const_iterator ptr = conf.Find( "operchan" ) ;
while( ptr != conf.end() && ptr->first == "operchan" )
	{
	operChans.push_back( ptr->second ) ;
	++ptr ;
	}

// Read out the client's message channel
msgChan = conf.Find( "msgchan" )->second ;


// Make sure that the msgChan is in the list of operchans
if( operChans.end() == find( operChans.begin(), operChans.end(), msgChan ) )
	{
	// Not found, add it to the list of operChans
	operChans.push_back( msgChan ) ;
	}


// Be sure to use all capital letters for the command name

RegisterCommand( new HELPCommand( this, "HELP", "[topic]"
	"\t\tObtain general help or help for a specific command",commandLevel::flg_HELP,false,false,true,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new INVITECommand( this, "INVITE", "<#channel> "
	"\t\tRequest an invitation to a channel",commandLevel::flg_INVITE,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new JUPECommand( this, "JUPE", "<servername> <reason> "
	"Jupe a server for the given reason.",commandLevel::flg_JUPE,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new MODECommand( this, "MODE", "<channel> <modes> "
	"Change modes on the given channel",commandLevel::flg_MODE,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new GLINECommand( this, "GLINE", "<user@host> <duration>[time units (s,d,h)] <reason> "
	"Gline a given user@host for the given reason",commandLevel::flg_GLINE,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new SCANGLINECommand( this, "SCANGLINE", "<mask> "
	"Search current network glines for glines matching <mask>",commandLevel::flg_SGLINE,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new REMGLINECommand( this, "REMGLINE", "<user@host> "
	"Remove the gline matching <mask>",commandLevel::flg_REMGLINE,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new TRANSLATECommand( this, "TRANSLATE", "<numeric>"
	"Translate a numeric into user information",commandLevel::flg_TRANS,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new WHOISCommand( this, "WHOIS", "<nickname>"
	"Obtain information on a given nickname",commandLevel::flg_WHOIS,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new KICKCommand( this, "KICK", "<channel> <nick> <reason>"
	"Kick a user from a channel",commandLevel::flg_KICK,false,false,false,operLevel::OPERLEVEL,false ) ) ;

//The following commands deals with operchans, if you want operchans just uncomment them
/*

RegisterCommand( new ADDOPERCHANCommand( this, "ADDOPERCHAN", "<channel>"
	"Add an oper channel",commandLevel::flg_ADDOPCHN,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new REMOPERCHANCommand( this, "REMOPERCHAN", "<channel>"
	"Remove an oper channel",commandLevel::flg_REMOPCHN,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new LISTOPERCHANSCommand( this, "LISTOPERCHANS",
	"List current IRCoperator only channels",commandLevel::flg_LOPCHN,false,false,false,operLevel::OPERLEVEL,false ) ) ;
*/	

RegisterCommand( new CHANINFOCommand( this, "CHANINFO", "<channel>"
	"Obtain information about a given channel",commandLevel::flg_CHINFO,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new LOGINCommand( this, "LOGIN", "<USER> <PASS> "
	"Authenticate with the bot",commandLevel::flg_LOGIN,false,true,true,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new DEAUTHCommand( this, "DEAUTH", ""
	"Deauthenticate with the bot",commandLevel::flg_DEAUTH,false,false,true,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new ADDUSERCommand( this, "ADDUSER", "<USER> <OPERTYPE> [SERVER*] <PASS> "
	"Add a new oper",commandLevel::flg_ADDNOP,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new REMUSERCommand( this, "REMUSER", "<USER> "
	"Remove an oper",commandLevel::flg_REMOP,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new ADDCOMMANDCommand( this, "ADDCOMMAND", "<USER> <COMMAND> "
	"Add a new command to an oper",commandLevel::flg_ADDCMD,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new REMCOMMANDCommand( this, "REMCOMMAND", "<USER> <COMMAND> "
	"Remove a command from oper",commandLevel::flg_DELCMD,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new NEWPASSCommand( this, "NEWPASS", "<PASSWORD> "
	"Change password",commandLevel::flg_NEWPASS,false,false,true,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new SUSPENDCommand( this, "SUSPEND", "<OPER> <DURATION> [-l level] <REASON>"
	"Suspend an oper",commandLevel::flg_SUSPEND,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new UNSUSPENDCommand( this, "UNSUSPEND", "<OPER> "
	"UnSuspend an oper",commandLevel::flg_UNSUSPEND,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new MODUSERCommand( this, "MODUSER", "<OPER> <OPTION> <NEWVALUE> [OPTION] [NEWVALUE] ... "
	"Modify an oper",commandLevel::flg_MODOP,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new MODERATECommand( this, "MODERATE", "<#Channel> "
	"Moderate A Channel",commandLevel::flg_MODERATE,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new UNMODERATECommand( this, "UNMODERATE", "<#Channel> "
	"UNModerate A Channel",commandLevel::flg_UNMODERATE,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new OPCommand( this, "OP", "<#Channel> <nick> [nick] .. "
	"Op user(s) on a Channel",commandLevel::flg_OP,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new DEOPCommand( this, "DEOP", "<#Channel> <nick> [nick] .. "
	"Deop user(s) on a Channel",commandLevel::flg_DEOP,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new LISTHOSTSCommand( this, "LISTHOSTS", "<oper> "
	"Shows an oper hosts list",commandLevel::flg_LISTHOSTS,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new CLEARCHANCommand( this, "CLEARCHAN", "<#chan> "
	"Removes all channel modes",commandLevel::flg_CLEARCHAN,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new ADDSERVERCommand( this, "ADDSERVER", "<Server> "
	"Add a new server to the bot database",commandLevel::flg_ADDSERVER,false,false,false,operLevel::OPERLEVEL,false ) ) ;
RegisterCommand( new LEARNNETCommand( this, "LEARNNET", ""
	"Update the servers database according to the current situation",commandLevel::flg_LEARNNET,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new REMSERVERCommand( this, "REMSERVER", "<Server name>"
	"Removes a server from the bot database",commandLevel::flg_REMSERVER,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new CHECKNETCommand( this, "CHECKNET", ""
	"Checks if all known servers are in place",commandLevel::flg_CHECKNET,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new LASTCOMCommand( this, "LASTCOM", "[number of lines to show]"
	"Post you the bot logs",commandLevel::flg_LASTCOM,false,false,false,operLevel::OPERLEVEL, true) ) ;
RegisterCommand( new FORCEGLINECommand( this, "FORCEGLINE", "<user@host> <duration>[time units] <reason> "
	"Gline a given user@host for the given reason",commandLevel::flg_FGLINE,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new EXCEPTIONCommand( this, "EXCEPTIONS", "(list / add / del) [host mask]"
	"Add connection exceptions on hosts",commandLevel::flg_EXCEPTIONS,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new LISTIGNORESCommand( this, "LISTIGNORES", ""
	"List the ignore list",commandLevel::flg_LISTIGNORES,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new REMOVEIGNORECommand( this, "REMIGNORE", "(nick/host)"
	" Removes a host/nick from the  ignore list",commandLevel::flg_REMIGNORE,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new LISTCommand( this, "LIST", "(glines/servers)"
	" Get all kinds of lists from the bot",commandLevel::flg_LIST,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new COMMANDSCommand( this, "COMMANDS", "<command> <option> <new value>"
	" Change commands options",commandLevel::flg_COMMANDS,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new GCHANCommand( this, "GCHAN", "#channel <length/-per> <reason>"
	" Set a BADCHAN gline",commandLevel::flg_GCHAN,false,false,false,operLevel::CODERLEVEL,true ) ) ;
RegisterCommand( new REMGCHANCommand( this, "REMGCHAN", "#channel "
	" Removes a BADCHAN gline",commandLevel::flg_GCHAN,false,false,false,operLevel::CODERLEVEL,true ) ) ;
RegisterCommand( new USERINFOCommand( this, "USERINFO", "<usermask/servermask> "
	" Get information about opers",commandLevel::flg_USERINFO,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new STATUSCommand( this, "STATUS", "Shows debug status "
        ,commandLevel::flg_STATUS,false,false,false,operLevel::CODERLEVEL,true ) ) ;
RegisterCommand( new SHUTDOWNCommand( this, "SHUTDOWN", " <REASON> Shutdown the bot "
        ,commandLevel::flg_SHUTDOWN,false,false,false,operLevel::CODERLEVEL,true ) ) ;
RegisterCommand( new SCANCommand( this, "SCAN", " -h <host> / -n <real name> [-v]"
		" Scans for all users which much a certain host / real name ",commandLevel::flg_SCAN,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new MAXUSERSCommand( this, "MAXUSERS", 
		"Shows the maximum number of online users ever recorded ",commandLevel::flg_MAXUSERS,false,false,false,operLevel::OPERLEVEL,true ) ) ;
RegisterCommand( new CONFIGCommand( this, "CONFIG", " "
		" Manages all kinds of configuration related values ",commandLevel::flg_CONFIG,false,false,false,operLevel::CODERLEVEL,true ) ) ;


loadCommands();
loadGlines();
if(!loadExceptions())
	{
	elog << "Error while loading exceptions!!!! , shutting down" << endl;
	::exit(1);
	}
	
loadUsers();
loadServers();
loadMaxUsers();
loadVersions();

connectCount = 0;
connectRetry = 5;
curUsers = 0;

#ifdef LOGTOHD
	initLogs();
#endif

}


ccontrol::~ccontrol()
{
// Deallocate each command handler
for( commandMapType::iterator ptr = commandMap.begin() ;
	ptr != commandMap.end() ; ++ptr )
	{
	delete ptr->second ;
	ptr->second = 0 ;
	}
commandMap.clear() ;
// Deallocate each gline entry
for(glineIterator GLptr = glineList.begin(); GLptr != glineList.end(); ++GLptr)
	{
	GLptr = glineList.erase(GLptr);
	}

}

// Register a command handler
bool ccontrol::RegisterCommand( Command* newComm )
{
assert( newComm != NULL ) ;

// Unregister the command handler first; prevent memory leaks
UnRegisterCommand( newComm->getName() ) ;

/*if(!UpdateCommandFromDb(newComm))
	elog << "Error cant find command "
	     << newComm->getName() 
	     << " In database"
	     << ends;*/


// Insert the new handler
return commandMap.insert( pairType( newComm->getName(), newComm ) ).second ;
}

bool ccontrol::UnRegisterCommand( const string& commName )
{
// Find the command handler
commandMapType::iterator ptr = commandMap.find( commName ) ;

// Was the handler found?
if( ptr == commandMap.end() )
	{
	// Nope
	return false ;
	}

// Deallocate the handler
//delete ptr->second ;

// Remove the handler
commandMap.erase( ptr ) ;

// Return success
return true ;
}

int ccontrol::BurstChannels()
{
//elog << "ccontrol::BurstChannels()\n" ;

// msgChan is an operChan as well, no need to burst it separately
for( vector< string >::size_type i = 0 ; i < operChans.size() ; i++ )
	{
	// Burst our channels
	MyUplink->JoinChannel( this, operChans[ i ], operChanModes ) ;

	// Receive events for this channel
	MyUplink->RegisterChannelEvent( operChans[ i ], this ) ;
	}

// Don't forget to call the base class BurstChannels() method
return xClient::BurstChannels() ;

}

// I don't really like doing this.
// In order for each of this bot's Command's to have a valid server
// pointer, this method must be overloaded and server must be
// explicitly set for each Command.
void ccontrol::ImplementServer( xServer* theServer )
{
for( commandMapType::iterator ptr = commandMap.begin() ;
	ptr != commandMap.end() ; ++ptr )
	{
	ptr->second->setServer( theServer ) ;
	}

expiredTimer = theServer->RegisterTimer(::time(0) + ExpiredInterval,this,NULL);
dbConnectionCheck = theServer->RegisterTimer(::time(0) + dbConnectionTimer,this,NULL);

if(SendReport)
	{
	struct tm Now = convertToTmTime(::time(0));
	time_t theTime = ::time(0) + ((24 - Now.tm_hour)*3600 - (Now.tm_min)*60) ; //Set the daily timer to 24:00
	postDailyLog = theServer->RegisterTimer(theTime, this, NULL); 
	}

if(checkGates)
	{
	gatesStatusCheck = theServer->RegisterTimer(::time(0) + 1,this,NULL);
	}
	
theServer->RegisterEvent( EVT_KILL, this );
theServer->RegisterEvent( EVT_QUIT, this );
theServer->RegisterEvent( EVT_NETJOIN, this );
theServer->RegisterEvent( EVT_BURST_CMPLT, this );
theServer->RegisterEvent( EVT_GLINE , this );
theServer->RegisterEvent( EVT_REMGLINE , this );
theServer->RegisterEvent( EVT_NICK , this );

theServer->RegisterEvent( EVT_NETBREAK, this );

xClient::ImplementServer( theServer ) ;
}

int ccontrol::OnPrivateMessage( iClient* theClient, const string& Message,
	bool )
{


// Tokenize the message
StringTokenizer st( Message ) ;


// Make sure there is a command present
if( st.empty() )
	{
	Notice( theClient, "Incomplete command" ) ;
	return 0 ;
	}

const string Command = string_upper( st[ 0 ] ) ;

ccUser* theUser = IsAuth(theClient);

if((!theUser) && !(theClient->isOper()))
	{ //We need to add the flood points for this user
	ccFloodData* floodData = (static_cast< ccUserData* >(
	theClient->getCustomData(this) ))->getFlood() ;
	if(floodData->addPoints(flood::MESSAGE_POINTS))
		{ //yes we need to ignore this user
		ignoreUser(floodData);
	
		MsgChanLog("[FLOOD MESSAGE]: %s has been ignored"
			,theClient->getNickName().c_str());
		}
	
	//return xClient::OnPrivateMessage( theClient, Message ) ;
	}


// Attempt to find a handler for this method.
commandIterator commHandler = findCommand( Command ) ;

// Was a handler found?
if( commHandler == command_end() )
	{
	// Nope, notify the client if he's authenticated
	//if(theUser)
		Notice( theClient, "Unknown command" ) ;
	return 0 ; 
	}

// Check if the user is logged in , and he got
// access to that command

int ComAccess = commHandler->second->getFlags();

if((!theUser) && (ComAccess) && !(theClient->isOper()))
	{
	return xClient::OnPrivateMessage( theClient, Message ) ;	
	}
if((!theUser) && !(ComAccess & commandLevel::flg_NOLOGIN) && (ComAccess))
	{//The user isnt authenticated, 
	 //and he must be to run this command
	if(theClient->isOper())
		Notice(theClient,"Sorry, but you must be authenticated to run this command");
	 return xClient::OnPrivateMessage( theClient, Message ) ;
	}	 	 

if((theUser) && (!theClient->isOper()) && (theUser->getNeedOp()))
	{
	Notice(theClient,
		"You must be operd up to use this command");
	}
else if( (ComAccess) && (theUser) && !(theUser->gotAccess(commHandler->second)))
	{
	Notice( theClient, "You dont have access to that command" ) ;
	}
else if(( (theUser) && isSuspended(theUser) ) && ( ComAccess ) )
		{
		Notice( theClient,
			"Sorry but you are suspended");
		}
else if(commHandler->second->getIsDisabled())
	{
	Notice(theClient,
		"Sorry, but this command is disabled");
	}
else 
	{
	// Log the command
	if(!commHandler->second->getNoLog()) //Dont log command which arent suppose to be logged
		{	
#ifndef LOGTOHD
		if(theUser)
			DailyLog(theUser,"%s",Message.c_str());
		else
		    	DailyLog(theClient,"%s",Message.c_str());
#else
		ccLog* newLog = new (std::nothrow) ccLog();
		newLog->Time = ::time(0);
		newLog->Desc = Message.c_str();
		newLog->Host = theClient->getNickUserHost().c_str();
		if(theUser)
			newLog->User = theUser->getUserName().c_str();
		else
			newLog->User = "Unknown";			
		newLog->CommandName = Command;
		DailyLog(newLog);
#endif
		}			
	// Execute the command handler
	commHandler->second->Exec( theClient, Message) ;
	}		
return xClient::OnPrivateMessage( theClient, Message ) ;
}

int ccontrol::OnServerMessage( iServer* Server, const string& Message,
	bool )
{
StringTokenizer st( Message ) ;

if(st.size() < 2)
    {
    return xClient::OnServerMessage(Server,Message);
    }

//This is kinda lame 
//TODO : add a server message parser
if(!strcasecmp(st[1],"351"))
	{
	if(st.size() < 5)
		{
		elog << "Invalid number of parameters on 351! :" << Message << endl;
	        return xClient::OnServerMessage(Server,Message);
		}
	ccServer* tmpServer = serversMap[Server->getName()];
	
	if(tmpServer)
		{
		tmpServer->setVersion(st[2] + " - " + st[4]);
		}
	else
		{
		serversMap.erase(serversMap.find(Server->getName()));
		}
	}
return xClient::OnServerMessage(Server,Message);
}

int ccontrol::Notice( const iClient* Target, const string& Message )
{
ccUser* tmpUser = IsAuth(Target);
if((tmpUser) && !(tmpUser->getNotice()))
	{
	return xClient::Message(Target,"%s",Message.c_str());
	}
return xClient::Notice(Target,Message);
}
         
int ccontrol::Notice( const iClient* Target, const char* Message, ... )
{

char buffer[ 1024 ] = { 0 } ;
va_list list;

va_start(list, Message);
vsnprintf(buffer, 1024, Message, list);
va_end(list);

ccUser* tmpUser = IsAuth(Target);
if((tmpUser) && !(tmpUser->getNotice()))
	{
	return xClient::Message(Target,"%s",buffer);
	}
return xClient::Notice(Target,"%s",buffer);
                        
}        

int ccontrol::OnCTCP( iClient* theClient, const string& CTCP
		, const string& Message ,bool Secure = false ) 
{

ccUser* theUser = IsAuth(theClient);
if((!theUser) && !(theClient->isOper()))
	{ //We need to add the flood points for this user
	ccFloodData* floodData = (static_cast< ccUserData* >(
	theClient->getCustomData(this) ))->getFlood() ;
	if(floodData->addPoints(flood::CTCP_POINTS))
		{ //yes we need to ignore this user
		ignoreUser(floodData);
		MsgChanLog("[FLOOD MESSAGE]: %s has been ignored"
			,theClient->getNickName().c_str());
		return false;
		}
	}
else
	{
	StringTokenizer st(CTCP);
	if(st[0] == "PING")
		{
		xClient::DoCTCP(theClient,CTCP,Message);
		}
	else if(st[0] == "GENDER")
		{
		return xClient::DoCTCP(theClient,CTCP,
		    "Thats a question i am still trying to find the answer to");
		}
	else if(st[0] == "SEX")
		{
		xClient::DoCTCP(theClient,CTCP,
		    "Sorry i am booked till the end of the year");
		}
	else if(st[0] == "POLICE")
		{
		xClient::DoCTCP(theClient,CTCP,
		    "Oh crap! where would i hide the drugs now?");
		}
	}
	
return true;
}

int ccontrol::OnEvent( const eventType& theEvent,
	void* Data1, void* Data2, void* Data3, void* Data4 )
{
switch( theEvent )
	{
	case EVT_QUIT:
	case EVT_KILL:
		{
		/*
		 *  The user disconnected,
		 *  remove his authentication,
		 *  and flood data, and login data
		 */
	
		iClient* tmpUser = (theEvent == EVT_QUIT) ?
			static_cast< iClient* >( Data1 ) :
			static_cast< iClient* >( Data2 ) ;
		--curUsers;
		if(checkClones)
			{
			string tIP = xIP( tmpUser->getIP()).GetNumericIP();
			if(--clientsIpMap[tIP] < 1)
				{
				clientsIpMap.erase(clientsIpMap.find(tIP));
				}
				string virtualHost = tmpUser->getDescription() + "@";
				int dots = 0;
				for(string::size_type ptr = 0;ptr < tIP.size(),dots < 3;++ptr)
					{
					if(tIP[ptr] == '.')
						{
						++dots;
						}
					virtualHost += tIP[ptr];
					}
				virtualHost += '*';
			if(--virtualClientsMap[virtualHost] < 1)
				{
				virtualClientsMap.erase(virtualClientsMap.find(virtualHost));
				}
			}
		ccUserData* UserData = static_cast< ccUserData* >(
		tmpUser->getCustomData(this))  ;
		tmpUser->removeCustomData(this);
		
		ccUser *TempAuth = UserData->getDbUser();
		if(TempAuth)
	    		{
			UserData->setDbUser(NULL);
			TempAuth->setClient(NULL);
			}
		ccFloodData *tempLogin = UserData->getFlood();
		if(tempLogin)
			{
			removeLogin(tempLogin);
			if(tempLogin->getIgnoredHost() != "")
				{
				tempLogin->setNumeric("0");
				tempLogin->resetLogins();
				}
			else
				{
				delete tempLogin;
				}		
			}
		delete UserData;
		break ;
		} // case EVT_KILL/case EVT_QUIT
	
	case EVT_NETJOIN:
		{
		inBurst = true;
		/*
		 * We need to update the servers table about the new
		 * server , and check if we know it
		 *
		 */
		if(dbConnected)
			{
			iServer* NewServer = static_cast< iServer* >( Data1);
			iServer* UplinkServer = static_cast< iServer* >( Data2);
			ccServer* CheckServer = getServer(NewServer->getName());
			if(!CheckServer)
				{    	
				MsgChanLog("Unknown server connected : %s Uplink : %s\n"
					    ,NewServer->getName().c_str(),UplinkServer->getName().c_str());
				}
			else 
				{
				CheckServer->setLastConnected(::time (0));
				CheckServer->setUplink(UplinkServer->getName());
				CheckServer->setLastNumeric(NewServer->getCharYY());
				CheckServer->setNetServer(NewServer);
				CheckServer->Update();
				Write("%s V :%s\n",getCharYYXXX().c_str(),NewServer->getCharYY());
				}
		
			}
		break;
		}
	case EVT_NETBREAK:
		{
		if(dbConnected)
			{
			iServer* NewServer = static_cast< iServer* >( Data1);
			string Reason = *(static_cast<string *>(Data3));
			
			if(!getUplink()->isJuped(NewServer))
				{
				ccServer* CheckServer = getServer(NewServer->getName());
	    	                if(CheckServer)
					{
					CheckServer->setSplitReason(Reason);
					CheckServer->setLastSplitted(::time(NULL));
					CheckServer->setNetServer(NULL);
					CheckServer->Update();
					}
				}
			}
		break;
		}
	case EVT_BURST_CMPLT:
		{
		inBurst = false;
		refreshGlines();
		burstGlines();
		checkMaxUsers();
		break;
		}	
	case EVT_GLINE:
		{
		if(!Data1) //TODO: find out how we get this
			{
			return 0;
			}

		Gline* newG = static_cast< Gline* >(Data1);

		ccGline* newGline = findGline(newG->getUserHost());
		if(!newGline)
			{
			newGline = new (std::nothrow) ccGline(SQLDb);
			assert (newGline != NULL);
			iServer* serverAdded = Network->findServer(newG->getSetBy());
			if(serverAdded)
				newGline->setAddedBy(serverAdded->getName());
			else
				newGline->setAddedBy("Unknown");
			}
		newGline->setAddedOn(::time(0));
		//newGline->setLastUpdated(::time(0));
		newGline->setHost(newG->getUserHost());
		newGline->setReason(newG->getReason());
		newGline->setExpires(newG->getExpiration());
		newGline->Insert();
		//need to load the id
		newGline->loadData(newGline->getHost());
		addGline(newGline);
		break;
		}
	case EVT_REMGLINE:
		{
		if(!Data1)  
			{
			return 0;
			}
			
		Gline* newG = static_cast< Gline* >(Data1);
		ccGline* newGline = findGline(newG->getUserHost());
		if(newGline)
			{
			remGline(newGline);
			newGline->Delete();
			delete newGline;
			}
		break;
		}
	case EVT_NICK:
		{
		bool glSet = false;
		iClient* NewUser = static_cast< iClient* >( Data1);
		curUsers++;
		if(!inBurst)
			{
			checkMaxUsers();
			}
		//Create our flood data for this user
		ccFloodData* floodData = new (std::nothrow) ccFloodData(NewUser->getCharYYXXX());
		assert( floodData != 0 ) ;
		ccUserData* UserData = new (std::nothrow) ccUserData(floodData);
		NewUser->setCustomData(this,
			static_cast< void* >( UserData ) );
		if(checkGates)
			{
			string tIP = xIP( NewUser->getIP()).GetNumericIP();
			if(strcasecmp(tIP,"0.0.0.0"))			
				{
				ccGate* tempGate = new (std::nothrow) ccGate(tIP,1080);
				if(inBurst)
					gatesWaitingQueue.push_back(tempGate);			
				else
					gatesWaitingQueue.push_front(tempGate);			
				tempGate = new (std::nothrow) ccGate(tIP,23);
				assert(tempGate != NULL);
				if(inBurst)
					gatesWaitingQueue.push_back(tempGate);			
				else
					gatesWaitingQueue.push_front(tempGate);			
				}			
			}
		if(dbConnected)
			{
			if(checkClones)
				{
				string tIP = xIP( NewUser->getIP()).GetNumericIP();
				if(strcasecmp(tIP,"0.0.0.0"))			
					{
					int CurConnections = ++clientsIpMap[tIP];
					if((CurConnections  > getExceptions("*@" + tIP)) 
					    && (CurConnections > getExceptions("*@"+NewUser->getInsecureHost())))
						{
						MsgChanLog("Glining %s , total  connections : %d\n"
						,tIP.c_str(),CurConnections);
						MsgChanLog(" IP Exception : %d , HOST Exception %d\n"						
						,getExceptions("*@" + tIP),getExceptions("*@" + NewUser->getInsecureHost()));

						glSet = true;
						ccGline *tmpGline;
						tmpGline = findGline("*@" + tIP); 
						if(!tmpGline)
							tmpGline = findGline("*@" + NewUser->getInsecureHost());
						if(!tmpGline)
							{
							tmpGline = new ccGline(SQLDb);
							tmpGline->setHost("*@" + tIP);
							tmpGline->setExpires(::time(0) + maxGlineLen);
							string Reason = "Automatically banned for excessive connections [- ";
							Reason += CurConnections;
							Reason += " -]";
							tmpGline->setReason("Automatically banned for excessive connections ");
							tmpGline->setAddedOn(::time(0));
							tmpGline->setAddedBy("Auto Gline");
							tmpGline->Insert();
							tmpGline->loadData(tmpGline->getHost());
							addGline(tmpGline);
							}
						MyUplink->setGline( nickName,
								tmpGline->getHost(),
								tmpGline->getReason(),
								tmpGline->getExpires() - ::time(0) , this) ;
						}	
					else
						{
						int dots = 0;
						string ipClass = "";
						for(string::size_type ptr = 0;ptr < tIP.size(),dots < 3;++ptr)
							{
							if(tIP[ptr] == '.')
								{
								++dots;
								}
							ipClass += tIP[ptr];
							}
						ipClass += '*';
						CurConnections = ++virtualClientsMap[NewUser->getDescription() + "@" + ipClass];
						if(CurConnections > getExceptions("*@"+ipClass))
							{
							MsgChanLog("Virtual clones for real name %s on %s, total connections %d\n",
							    NewUser->getDescription().c_str()
							    ,ipClass.c_str()
							    ,CurConnections);
							}
						}
					}
				}
			if(!glSet) 
				{	
				ccGline * tempGline = findMatchingGline(NewUser->getUserName() + '@' + NewUser->getInsecureHost());
				if(!tempGline)
					{
					string tIP = xIP( NewUser->getIP()).GetNumericIP();
					tempGline = findMatchingGline(NewUser->getUserName() + '@' + tIP);
					}
				if((tempGline) && (tempGline->getExpires() > ::time(0)))
					{
					glSet = true;
					if(!inBurst)
						{
						MyUplink->setGline(tempGline->getAddedBy()
						,tempGline->getHost(),tempGline->getReason()
						,tempGline->getExpires() - ::time(0),this);
						}
					else
						{
						string* tServer = new (std::nothrow) string(NewUser->getCharYY());
						assert(tServer != NULL);
						tempGline->addBurst(tServer);
						}
					//unSetRemoving();
					}
				}			
			}
		break;
		}
	} // switch()

return 0;

return xClient::OnEvent( theEvent, Data1, Data2, Data3, Data4 ) ;
}

int ccontrol::OnChannelEvent( const channelEventType& theEvent,	Channel* theChan,
	void* Data1, void* Data2, void* Data3, void* Data4 )
{

switch( theEvent )
	{
	case EVT_JOIN:
		if( !isOperChan( theChan ) )
			{
			// We really don't care otherwise
			// Note, this shouldn't happen
			break ;
			}

		iClient* theClient = static_cast< iClient* >( Data1 ) ;
		if( theClient->isOper() )
			{
			Op( theChan, theClient ) ;
			}
		break ;
	}

// Call the base class OnChannelEvent()
return xClient::OnChannelEvent( theEvent, theChan,
	Data1, Data2, Data3, Data4 ) ;
}

int ccontrol::OnTimer(xServer::timerID timer_id, void*)
{

if (timer_id ==  postDailyLog)
	{ 
	// Create the lastcom report 
	CreateReport(::time(0) - 24*3600,::time(0));

	// Email the report to the abuse team
	MailReport(AbuseMail.c_str(),"Report.log");

	/* Refresh Timers */			
	time_t theTime = time(NULL) + 24*3600; 
	postDailyLog = MyUplink->RegisterTimer(theTime, this, NULL); 
	}
else if (timer_id == expiredTimer)
	{
	refreshGlines();
	refreshIgnores();
	refreshSuspention();
	expiredTimer = MyUplink->RegisterTimer(::time(0) + ExpiredInterval,
		this,NULL);
	}
else if(timer_id == gatesStatusCheck)
	{
	if(checkGates)
		{
		GatesCheck();
		gatesStatusCheck = MyUplink->RegisterTimer(::time(0) + 2,this,NULL);
		}
	}
else if(timer_id == dbConnectionCheck)
	{
	checkDbConnection();
	dbConnectionCheck = MyUplink->RegisterTimer(::time(0) + dbConnectionTimer,this,NULL);
	}
	
return true;
}

int ccontrol::OnConnect()
{
iServer* tmpServer = Network->findServer(getUplink()->getUplinkCharYY());
ccServer* tServer = getServer(tmpServer->getName());
if(tServer)
	{
	tServer->setNetServer(tmpServer);
	Write("%s V :%s\n",getCharYYXXX().c_str(),tmpServer->getCharYY());
	}

return xClient::OnConnect();
}

bool ccontrol::isOperChan( const string& theChan ) const
{
vector< string >::const_iterator ptr = operChans.begin(),
	end = operChans.end() ;
while( ptr != end )
	{
	if( !strcasecmp( *ptr, theChan ) )
		{
		return true ;
		}
	++ptr ;
	}
return false ;
}

bool ccontrol::isOperChan( const Channel* theChan ) const
{
assert( theChan != 0 ) ;

return isOperChan( theChan->getName() ) ;
}

// This method does NOT add the channel to any internal tables
bool ccontrol::Join( const string& chanName, const string& chanModes,
	time_t joinTime, bool getOps )
{
if( isOnChannel( chanName ) )
	{
	// Already on this channel
	return true ;
	}
bool result = xClient::Join( chanName, chanModes, joinTime, getOps ) ;
if( result )
	{
	MyUplink->RegisterChannelEvent( chanName, this ) ;
	}

operChans.push_back( chanName ) ;
return result ;
}

bool ccontrol::Part( const string& chanName )
{
bool foundChannel = false ;
for( vector< string >::iterator ptr = operChans.begin() ;
	ptr != operChans.end() ; ++ptr )
	{
	if( !strcasecmp( (*ptr).c_str(), chanName.c_str() ) )
		{
		operChans.erase( ptr ) ;
		foundChannel = true ;
		break ;
		}
	} // for()

if( !foundChannel )
	{
	// The bot isn't on the channel
	return false ;
	}

bool result = xClient::Part( chanName ) ;
if( result )
	{
	MyUplink->UnRegisterChannelEvent( chanName, this ) ;
	}

return result ;
}

bool ccontrol::Kick( Channel* theChan, iClient* theClient,
	const string& reason )
{
assert( theChan != NULL ) ;

if( !isOnChannel( theChan->getName() ) )
	{
	return false ;
	}

return xClient::Kick( theChan, theClient, reason ) ;
}

bool ccontrol::addOperChan( const string& chanName )
{
return addOperChan( chanName, operChanReason ) ;
}

bool ccontrol::addOperChan( const string& chanName, const string& reason )
{
if( isOperChan( chanName ) )
	{
	return false ;
	}

xClient::Join( chanName, operChanModes, 0, true ) ;
MyUplink->RegisterChannelEvent( chanName, this ) ;
operChans.push_back( chanName ) ;

Channel* theChan = Network->findChannel( chanName ) ;
if( NULL == theChan )
	{
	elog	<< "ccontrol::addOperChan> Unable to find channel: "
		<< chanName << endl ;
	return false ;
	}

// Kick any users from the channel that aren't opers
vector< iClient* > clientsToKick ;
for( Channel::const_userIterator ptr = theChan->userList_begin() ;
	ptr != theChan->userList_end() ; ++ptr )
	{
	// Dont kick opers and +k ppl
	if(( !ptr->second->isOper() ) && ( !ptr->second->getClient()->getMode(iClient::MODE_SERVICES) ))
		{
		clientsToKick.push_back( ptr->second->getClient() ) ;
		}
	}

if( !clientsToKick.empty() )
	{
	xClient::Kick( theChan, clientsToKick, reason ) ;
	}

// TODO: set operChanModes

return true ;
}

bool ccontrol::removeOperChan( const string& chanName )
{
// Part() will remove the channel from this client's tables.
Part( chanName ) ;

return true ;
}

ccUser* ccontrol::IsAuth( const iClient* theClient ) 
{
return (static_cast< ccUserData* >(theClient->getCustomData(this)))->getDbUser() ;
}

ccUser* ccontrol::IsAuth( const string& Numeric ) 
{
return IsAuth(Network->findClient(Numeric));
}


bool ccontrol::AddOper (ccUser* Oper)
{
static const char *Main = "INSERT into opers (user_name,password,access,saccess,last_updated_by,last_updated,flags,server,isSuspended,suspend_expires,suspended_by,suspend_level,suspend_reason,isUhs,isOper,isAdmin,isSmt,isCoder,GetLogs,NeedOp,Notice) VALUES ('";

if(!dbConnected)
	{
	return false;
	}
strstream theQuery;
theQuery	<< Main
		<< Oper->getUserName() <<"','"
		<< Oper->getPassword() << "',"
		<< Oper->getAccess() << ","
		<< Oper->getSAccess() << ",'"
		<< Oper->getLast_Updated_by()
		<< "',now()::abstime::int4,"
		<< Oper->getFlags() << ",'"
		<< Oper->getServer() 
		<< "' ," 
		<< (Oper->getIsSuspended() ? "'t'" : "'n'") 
		<< "," << Oper->getSuspendExpires()
		<< ",'" << Oper->getSuspendedBy()
		<< "'," << Oper->getSuspendLevel()
		<< ",'" << Oper->getSuspendReason()
		<< "'," << (Oper->isUhs() ? "'t'" : "'n'")
		<< "," << (Oper->isOper() ? "'t'" : "'n'")
		<< "," << (Oper->isAdmin() ? "'t'" : "'n'")
		<< "," << (Oper->isSmt() ? "'t'" : "'n'")
		<< "," << (Oper->isCoder() ? "'t'" : "'n'")
		<< "," << (Oper->getLogs() ? "'t'" : "'n'")
		<< "," << (Oper->getNeedOp() ? "'t'" : "'n'")
		<< "," << (Oper->getNotice() ? "'t'" : "'n'")
		<< ")"
		<< ends;

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_COMMAND_OK == status ) 
	{
	if(!Oper->loadData(Oper->getUserName()))
		return false;
	usersMap[Oper->getUserName()] = Oper;
	return true;
	}

elog	<< "ccontrol::AddOper> SQL Failure: "
	<< SQLDb->ErrorMessage()
	<< endl ;
return false;
}

bool ccontrol::DeleteOper (const string& Name)
{

if(!dbConnected)
	{
	return false;
	}

ExecStatusType status;
ccUser* tUser = usersMap[Name];
//Delete the user hosts
if(tUser)
    {
    static const char *tMain = "DELETE FROM hosts WHERE User_Id = ";    
    strstream HostQ;
    HostQ << tMain;
    HostQ << tUser->getID();
    HostQ << ends;
    status = SQLDb->Exec( HostQ.str() ) ;
elog	<< "ccontrol::DeleteOper> "
	<< HostQ.str()
	<< endl; 

    delete[] HostQ.str();
if( PGRES_COMMAND_OK != status ) 
	{
	elog	<< "ccontrol::DeleteOper> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false;
	}
	usersMap.erase(usersMap.find(Name));
    }    

static const char *Main = "DELETE FROM opers WHERE lower(user_name) = '";

strstream theQuery;
theQuery	<< Main
		<< removeSqlChars(Name)
		<< "'"
		<< ends;

elog	<< "ccontrol::DeleteOper> "
	<< theQuery.str()
	<< endl; 

status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_COMMAND_OK == status ) 
	{
	return true;
	}
else
	{
	elog	<< "ccontrol::DeleteOper> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false;
	}
}


int ccontrol::getCommandLevel( const string& Command)
{
commandIterator commHandler = findCommand( Command ) ;

// Was a handler found?
if( commHandler != command_end() )
	{
	return commHandler->second->getFlags() ;
	}

return -1 ;
}	

bool ccontrol::AuthUser( ccUser* TempUser,iClient* tUser)
{
ccUserData* UserData= static_cast<ccUserData*>(tUser->getCustomData(this));
UserData->setDbUser(TempUser);
TempUser->setClient(tUser);

return true;
}    

bool ccontrol::deAuthUser( ccUser* tUser)
{

const iClient* tClient = tUser->getClient();
if(tClient)
	{
	(static_cast<ccUserData*>(tClient->getCustomData(this)))->setDbUser(NULL);
	tUser->setClient(NULL);
	}
		
return true;
}

bool ccontrol::UserGotMask( ccUser* user, const string& Host )
{
static const char *Main = "SELECT host FROM hosts WHERE user_id = ";

if(!dbConnected)
	{
	return false;
	}

strstream theQuery;
theQuery	<< Main
		<< user->getID()
		<< ';'
		<< ends;

elog	<< "ccontrol::UserGotMask> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK != status )
	{
	elog	<< "ccontrol::UserGotMask> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;

	return false ;
	}

for( int i = 0 ; i < SQLDb->Tuples() ; i++ )
	{
	if(match(SQLDb->GetValue(i,0),Host) == 0)
		{
		return true ;
		}
	}

return false ;
}

bool ccontrol::UserGotHost( ccUser* user, const string& Host )
{
static const char *Main = "SELECT host FROM hosts WHERE user_id = ";

if(!dbConnected)
	{
	return false;
	}

strstream theQuery;
theQuery	<< Main
		<< user->getID()
		<< ';'
		<< ends;

elog	<< "ccontrol::UserGotHost> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK == status )
	{
	for( int i = 0 ; i < SQLDb->Tuples() ; i++ )
		{
		if(!strcasecmp(SQLDb->GetValue(i,0),Host.c_str()))
			{
			return true ;
			}
		}
	}
return false ;
}

string ccontrol::CryptPass( const string& pass )
{
StringTokenizer st( pass ) ;
	
const char validChars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789.$*_";
	
string salt;

for ( unsigned short int i = 0 ; i < 8 ; i++ ) 
	{ 
	int randNo = 1+(int) (64.0*rand()/(RAND_MAX+1.0));
	salt += validChars[randNo]; 
	} 

/* Work out a MD5 hash of our salt + password */

md5	hash; // MD5 hash algorithm object.
md5Digest digest; // MD5Digest algorithm object.
 
strstream output;
string newPass;
newPass = salt + st.assemble(0);

hash.update( (const unsigned char *)newPass.c_str(), newPass.size() );
hash.report( digest );
	
/* Convert to Hex */
int data[ MD5_DIGEST_LENGTH ] = { 0 } ;
for( size_t ii = 0; ii < MD5_DIGEST_LENGTH; ii++ )
	{
	data[ii] = digest[ii];
	}

output << hex;
output.fill('0');
for( size_t ii = 0; ii < MD5_DIGEST_LENGTH; ii++ )
	{
	output << setw(2) << data[ii];
	}
output << ends;

return string( salt + output.str() );
}

bool ccontrol::validUserMask(const string& userMask) const
{

// Check that a '!' exists, and that the nickname
// is no more than 9 characters
StringTokenizer st1( userMask, '!' ) ;
if( (st1.size() != 2) || (st1[ 0 ].size() > 9) )
	{
	return false ;
	}

// Check that a '@' exists and that the username is
// no more than 12 characters
StringTokenizer st2( st1[ 1 ], '@' ) ;

if( (st2.size() != 2) || (st2[ 0 ].size() > 12) )
	{
	return false ;
	}

// Be sure that the hostname is no more than 128 characters
if( st2[ 1 ].size() > 128 )
	{
	return false ;
	}

// Tests have passed
return true ;
}

bool ccontrol::AddHost( ccUser* user, const string& host )
{

if(!dbConnected)
	{
	return false;
	}

static const char *Main = "INSERT into hosts (user_id,host) VALUES (";

strstream theQuery;
theQuery	<< Main
		<< user->getID() <<",'"
		<< host << "')"
		<< ends;

elog	<< "ccontrol::AddHost> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_COMMAND_OK == status ) 
	{
	return true;
	}
else
	{
	elog	<< "ccontrol::AddHost> SQL Error: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false;
	}
}

bool ccontrol::DelHost( ccUser* user, const string& host )
{

if(!dbConnected)
	{
	return false;
	}

static const char *Main = "DELETE FROM hosts WHERE user_id = ";

strstream theQuery;
theQuery	<< Main
		<< user->getID()
		<< " And host = '"
		<< host << "'"
		<< ends;

elog	<< "ccontrol::DelHost> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_COMMAND_OK == status ) 
	{
	return true;
	}
else
	{
	elog	<< "ccontrol::DelHost> SQL Error: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false;
	}
}

bool ccontrol::listHosts( ccUser* User, iClient* theClient )
{
static const char* queryHeader
	= "SELECT host FROM hosts WHERE user_id =  ";

if(!dbConnected)
	{
	return false;
	}

strstream theQuery;
theQuery	<< queryHeader 
		<< User->getID()
		<< ends;

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK != status )
	{
	elog	<< "LISTHOSTS> SQL Error: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false ;
	}

// SQL Query succeeded
Notice(theClient,"Host list for %s",User->getUserName().c_str());
for (int i = 0 ; i < SQLDb->Tuples(); i++)
	{
	Notice(theClient,"%s",SQLDb->GetValue(i, 0));
	}
return true;
}	
	    

bool ccontrol::GetHelp( iClient* user, const string& command )
{
static const char *Main = "SELECT line,help FROM help WHERE lower(command) = '";

if(!dbConnected)
	{
	return false;
	}

strstream theQuery;
theQuery	<< Main
		<< string_lower(removeSqlChars(command))
		<< "' ORDER BY line"
		<< ends;

elog	<< "ccontrol::GetHelp> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK == status )
	{
	if(SQLDb->Tuples() > 0 )
		{
		DoHelp(user);
		}
	else
		{
		Notice( user,
			"Couldnt find help for %s",
			command.c_str());
		}
	return true;
	}
else
	{
	elog	<< "ccontrol::GetHelp> SQL Error: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false;	    
	}
}

bool ccontrol::GetHelp( iClient* user, const string& command , const string& subcommand)
{
static const char *Main = "SELECT line,help FROM help WHERE lower(command) = '";

if(!dbConnected)
	{
	return false;
	}

strstream theQuery;
theQuery	<< Main
		<< string_lower(removeSqlChars(command))
		<< "' and lower(subcommand) = '"
		<< string_lower(removeSqlChars(subcommand))
		<< "' ORDER BY line"
		<< ends;

elog	<< "ccontrol::GetHelp> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK == status )
	{
	if(SQLDb->Tuples() > 0 )
		{
		DoHelp(user);
		}
	else
		{
		Notice( user,
			"Couldnt find help for %s",
			command.c_str());
		}
	return true;
	}
else
	{
	elog	<< "ccontrol::GetHelp> SQL Error: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false;	    
	}
}


void ccontrol::DoHelp(iClient* theClient)
{
for( int i = 0 ; i < SQLDb->Tuples() ; i++ )
	{
	string commInfo = replace(
		SQLDb->GetValue( i, 1 ),
		"$BOT$",
		nickName ) ;
	Notice(theClient,"%s",commInfo.c_str());
	} // for()
}
	
string ccontrol::replace( const string& srcString,
	const string& from,
	const string& to )
{
string retMe( srcString ) ;
string::size_type beginPos = 0 ;

while((beginPos = retMe.find(from)) <= retMe.size())
	{
	retMe.replace(beginPos,from.size(),to);
	}

return retMe ;

}

ccUser* ccontrol::GetOper( const string Name)
{
ccUser* tempUser = usersMap[Name];
if(!tempUser)
	{
	usersMap.erase(usersMap.find(Name));
	}
return tempUser;
}

ccUser* ccontrol::GetOper( unsigned int ID)
{
usersIterator tIterator = usersMap.begin();
while(tIterator != usersMap.end())
	{
	ccUser* tUser = tIterator->second;;
	if(tUser->getID() == ID)
		{
		return tIterator->second;
		}
	++tIterator;
	}
return NULL;
}

bool ccontrol::addGline( ccGline* TempGline)
{

ccGline *theGline = 0;
bool addedAlready = false;
for(glineIterator ptr = glineList.begin(); ptr != glineList.end();)
	{
	theGline = *ptr;
	if(theGline->getHost() == TempGline->getHost()) 
		{
		if(TempGline != theGline)  //Make sure we are not deleting the one we need to add
			{
			ptr = glineList.erase(ptr);
			delete theGline;
			}
		else 
			{
			addedAlready = true;
			++ptr;
			}		
		}
	else
		++ptr;
	}
if(!addedAlready) //if we found the gline we need to add, no need to add it			
    glineList.push_back( TempGline ) ;
return true;
}    

bool ccontrol::remGline( ccGline* TempGline)
{
glineList.erase( std::find( glineList.begin(),
	glineList.end(),
	TempGline ) ) ;
return true;
}

ccGline* ccontrol::findMatchingGline( const string& Host )
{
ccGline *theGline = 0;
for(glineIterator ptr = glineList.begin(); ptr != glineList.end(); ++ptr)
	{
	theGline = *ptr;
	if(match(theGline->getHost(),Host) == 0) 
		{
    		if(theGline->getExpires() > ::time(0))
			{
			return theGline;
			}
		}
	}

return NULL ;
}

ccGline* ccontrol::findGline( const string& HostName )
{
ccGline *theGline;
for(glineIterator ptr = glineList.begin(); ptr != glineList.end();++ptr)
	{
	theGline = *ptr;
    	if(!strcasecmp(theGline->getHost(),HostName))
		if(theGline->getExpires() > ::time(0))
			return theGline;
	}

return NULL ;
}

struct tm ccontrol::convertToTmTime(time_t NOW)
{
return *gmtime(&NOW);
}

// TODO: This method should never exist
char *ccontrol::convertToAscTime(time_t NOW)
{
time_t *tNow = &NOW;
struct tm* Now = gmtime(tNow);
char *ATime = asctime(Now);
ATime[strlen(ATime)-1] = '\0';
return ATime;
}

bool ccontrol::MsgChanLog(const char *Msg, ... ) 
{
if(!Network->findChannel(msgChan))
	{
	return false;
	}

char buffer[ 1024 ] = { 0 } ;
va_list list;

va_start( list, Msg ) ;
vsprintf( buffer, Msg, list ) ;
va_end( list ) ;

xClient::Notice((Network->findChannel(msgChan))->getName(),"%s",buffer);
usersIterator uIterator;
ccUser* tempUser;
for( uIterator = usersMap.begin();uIterator != usersMap.end();++uIterator)
        {
        tempUser = uIterator->second;
	if((tempUser) && (tempUser->getLogs() ) && (tempUser->getClient()))
                { 
                Notice(tempUser->getClient(),"%s",buffer);
                }
	}
return true;
}

#ifndef LOGTOHD
bool ccontrol::DailyLog(ccUser* Oper, const char *Log, ... )
{

if(!dbConnected)
	{
	return false;
	}

char buffer[ 1024 ] ;
memset( buffer, 0, 1024 ) ;

va_list list;

va_start( list, Log ) ;
vsprintf( buffer, Log, list ) ;
va_end( list ) ;

iClient* theClient = 0 ;
if(Oper)
	{	
	theClient = Network->findClient(Oper->getNumeric());
	}

buffer[ 512 ] = 0 ;
static const char *Main = "INSERT into comlog (ts,oper,command) VALUES (now()::abstime::int4,'";
StringTokenizer st(buffer);
commandIterator tCommand = findCommand((string_upper(st[0])));
string log;
if(tCommand != command_end())
	{
	if(!strcasecmp(tCommand->second->getRealName(),"LOGIN"))
		{
		log.assign(string("LOGIN ") + st[1] + string(" *****"));
		}
	else if(!strcasecmp(tCommand->second->getRealName(),"NEWPASS"))
		{
		log.assign("NEWPASS *****");
		}
	else if(!strcasecmp(tCommand->second->getRealName(),"MODUSER"))
		{
		if(st.size() > 2)
			{
			log.assign("MODUSER " + st[1] + " ");
			unsigned int place = 2;
			while(place < st.size())
				{
				if(!strcasecmp(st[place],"-p"))
					{
					log.append(" -p ******");
					place+=2;
					}
				else	
					{
					log.append(" " + st[place]);
					place++;
					}
				}
			}
		}
	else if(!strcasecmp(tCommand->second->getRealName(),"ADDUSER"))
		{
		if(st.size() > 3)
			{
			log.assign("ADDUSER " + st[1] + string(" ") + st[2]+ " *****");
			}
		}
	else
		{
		log.assign(buffer);
		}
	}
else
	{
	log.assign(buffer);
	}
strcpy(buffer,log.c_str());
					
strstream theQuery;
theQuery	<< Main;
if(Oper)
	{
	theQuery << Oper->getUserName();
	}
else
	{
	theQuery << "Unknown";
	}
theQuery	<< " (" << removeSqlChars(theClient->getNickUserHost()) <<")','"
		<< removeSqlChars(buffer) << "')"
		<< ends;

elog	<< "ccontrol::DailyLog> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_COMMAND_OK == status ) 
	{
	return true;
	}
else
	{
	elog	<< "ccontrol::DailyLog> SQL Error: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false;
	}

return true;
}

bool ccontrol::DailyLog(iClient* theClient, const char *Log, ... )
{

if(!dbConnected)
	{
	return false;
	}

char buffer[ 1024 ] = { 0 } ;
va_list list;

va_start( list, Log ) ;
vsprintf( buffer, Log, list ) ;
va_end( list ) ;
buffer[512]= '\0';
static const char *Main = "INSERT into comlog (ts,oper,command) VALUES (now()::abstime::int4,'";
StringTokenizer st(buffer);
commandIterator tCommand = findCommand((string_upper(st[0])));
string log;
if(tCommand != command_end())
	{
	if(!strcasecmp(tCommand->second->getRealName(),"LOGIN"))
		{
		log.assign(string("LOGIN ") + st[1] + string(" *****"));
		}
	else if(!strcasecmp(tCommand->second->getRealName(),"NEWPASS"))
		{
		log.assign("NEWPASS *****");
		}
	else if(!strcasecmp(tCommand->second->getRealName(),"MODUSER"))
		{
		if(st.size() > 2)
			{
			log.assign("MODUSER " + st[1] + " ");
			unsigned int place = 2;
			while(place < st.size())
				{
				if(!strcasecmp(st[place],"-p"))
					{
					log.append(" -p ******");
					place+=2;
					}
				else	
					{
					log.append(" " + st[place]);
					place++;
					}
				}
			}
		}
	else if(!strcasecmp(tCommand->second->getRealName(),"ADDUSER"))
		{
		if(st.size() > 3)
			{
			log.assign("ADDUSER " + st[1] + string(" ") + st[2]+ " *****");
			}
		}
	else
		{
		log.assign(buffer);
		}
	}
else
	{
	log.assign(buffer);
	}
strcpy(buffer,log.c_str());
					
strstream theQuery;
theQuery	<< Main
		<< "Unknown"
		<< " (" << removeSqlChars(theClient->getNickUserHost()) <<")','"
		<< removeSqlChars(buffer) << "')"
		<< ends;

elog	<< "ccontrol::DailyLog> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_COMMAND_OK == status ) 
	{
	return true;
	}
else
	{
	elog	<< "ccontrol::DailyLog> SQL Error: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false;
	}

return true;
}

#else
bool ccontrol::DailyLog(ccLog* newLog)
{
commandIterator tCommand = findCommand(newLog->CommandName);
string log;
StringTokenizer st(newLog->Desc);
if(tCommand != command_end())
	{
	if(!strcasecmp(tCommand->second->getRealName(),"LOGIN"))
		{
		log.assign(string("LOGIN ") + st[1] + string(" *****"));
		}
	else if(!strcasecmp(tCommand->second->getRealName(),"NEWPASS"))
		{
		log.assign("NEWPASS *****");
		}
	else if(!strcasecmp(tCommand->second->getRealName(),"MODUSER"))
		{
		if(st.size() > 2)
			{
			log.assign("MODUSER " + st[1] + " ");
			unsigned int place = 2;
			while(place < st.size())
				{
				if(!strcasecmp(st[place],"-p"))
					{
					log.append(" -p ******");
					place+=2;
					}
				else	
					{
					log.append(" " + st[place]);
					place++;
					}
				}
			}
		}
	else if(!strcasecmp(tCommand->second->getRealName(),"ADDUSER"))
		{
		if(st.size() > 3)
			{
			log.assign("ADDUSER " + st[1] + string(" ") + st[2]+ " *****");
			}
		}
	else
		{
		log.assign(newLog->Desc);
		}
	}
else
	{
	log.assign(newLog->Desc);
	}

newLog->Desc = log;
if(!LogFile.is_open())
	{
	LogFile.open(LogFileName.c_str(),ios::in|ios::out);
	//LogFile.setbuf(NULL,0);
	}
if(LogFile.bad())
	{//There was a problem in opening the log file
	MsgChanLog("Error while logging to the logs file %s!\n",LogFileName.c_str());
	return true;
	}

LogFile.seekp(0,ios::end);
if(!newLog->Save(LogFile))
	{
	MsgChanLog("Error while logging to the log file!\n");
	}
addLog(newLog);
LogFile.close();

if(NumOfLogs > 0)
	NumOfLogs++;
return true;
}

#endif

bool ccontrol::CreateReport(time_t From, time_t Til)
{

if(!dbConnected)
	{
	return false;
	}

static const char* queryHeader = "SELECT * FROM comlog where ts >";
strstream theQuery;
theQuery 	<< queryHeader 
		<< From
		<< " AND ts < "
		<< Til
		<< " ORDER BY ts DESC"
		<< ends;
	
elog	<< "ccontrol::CreateReport> " 
	<< theQuery.str() 
	<< endl;
	
ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK != status )
	{
	elog	<< "ccontrol::CreateReport> SQL Error: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false ;
	}

// SQL Query succeeded
ofstream tLogFile;
tLogFile.open("Report.log",ios::out);
if(!tLogFile)
	{
	return false;
	}

tLogFile << "ccontrol log for command issued between "
	<< convertToAscTime(From)
	<< " and up til "
	<< convertToAscTime(Til)
	<< endl;

for (int i = 0 ; i < SQLDb->Tuples(); i++)
	{
	tLogFile	<< "[ "
		<< convertToAscTime(atoi(SQLDb->GetValue(i, 0)))
		<< " - "
		<< SQLDb->GetValue(i,1)
		<< " ] "
		<< SQLDb->GetValue(i,2)
		<< endl;
	}

tLogFile << "End of debug log"
	<< endl ;

tLogFile.close();
return true;
}

bool ccontrol::MailReport(const char *MailTo, char *ReportFile)
{

ifstream Report;
Report.open(Sendmail_Path.c_str(),ios::in);

if(!Report)
	{
	MsgChanLog("Error cant find sendmail, check the conf setting and"
		" try again\n");
	return false;
	}
Report.close();

Report.open(ReportFile,ios::in);
if(!Report)
	{
	MsgChanLog("Error while sending report\n");
	return false;
	}

ofstream TMail;
TMail.open("Report.mail",ios::out);
if(!TMail)
	{
	MsgChanLog("Error while sending report\n");
	return false;
	}
TMail << "Subject : CControl command log report\n";

string line ;

while( std::getline( Report, line ) )
	{
	TMail	<< line
		<< endl ;
	}
TMail.close();
Report.close();
char SendMail[256] = { 0 };

sprintf(SendMail, "%s -f %s %s < Report.mail\n",
	Sendmail_Path.c_str(),
	CCEmail.c_str(),
	MailTo);

system(SendMail);
return true;
}


int ccontrol::checkGline(const string Host,unsigned int Len,unsigned int &Affected)
{

const unsigned int isWildCard = 0x01;
const unsigned int isIP = 0x02;
unsigned int Mask = 0;
unsigned int Dots = 0;
unsigned int GlineType = isIP;
bool ParseEnded = false;
int retMe = 0;
string::size_type pos = Host.find_first_of('@');
string Hostname = Host.substr(pos+1);
if(Len >  gline::MFU_TIME)  //Check for maximum time
	retMe |=  gline::BAD_TIME;
if((signed int) Len < 0)
	retMe |=  gline::NEG_TIME;

for(string::size_type pos = 0; pos < Hostname.size();++pos)
	{
	if(Hostname[pos] =='.')
		{
		Dots++;
		if((GlineType & (isWildCard | isIP)) == isIP)
			Mask+=8; //Keep track of the mask
		}
	else if((Hostname[pos] =='*') || (Hostname[pos] == '?'))
		GlineType |= isWildCard;
	else if(Hostname[pos] == '/')
		{
		//For now not handled, return a bad host
		retMe |=  gline::BAD_HOST;
		/*if(!(GlineType & isIP)) //must be an ip to specify 
			return  gline::BAD_HOST;
		 Mask = atol((Host.substr(++pos)).c_str());
		 if(!(Mask) || (Mask > 32))
			return  gline::BAD_HOST;
		 if(Mask < 32)
			GlineType |= isWildCard;
		 ParseEnded = true;			
		 break;*/
		 
		 }
	else if((Hostname[pos] > '9') || (Hostname[pos] < '0')) 
		GlineType &= ~isIP;
	}


Affected = Network->countMatchingUserHost(Host); //Calculate the number of affected
if((Dots > 3) && (GlineType & isIP)) //IP addy cant have more than 3 dots
	retMe |=  gline::BAD_HOST;
if((GlineType & (isIP || isWildCard) == isIP) && !(ParseEnded))
	Mask +=8; //Add the last mask count if needed
if((GlineType & isIP) && (Mask < 24))
	retMe |=  gline::HUH_NO_HOST;  //Its too wide
if(!(GlineType & isIP) && (Dots < 2) && (GlineType & isWildCard))
	retMe |=  gline::HUH_NO_HOST; //Wildcard gline must have atleast 2 dots
if(Affected >  gline::MFGLINE_USERS) 
	retMe |=  gline::FU_NEEDED_USERS; //This gline must be set with -fu flag
if(Len >  gline::MFGLINE_TIME)
	retMe |=  gline::FU_NEEDED_TIME;
if(Len >  gline::MGLINE_TIME)
	retMe |=  gline::FORCE_NEEDED_TIME;
if(GlineType & (isWildCard & (Len >  gline::MGLINE_WILD_TIME)))
	retMe |=  gline::FORCE_NEEDED_WILDTIME;
if(!retMe)
	retMe =  gline::GLINE_OK;
return retMe;
}

bool ccontrol::isSuspended(ccUser *theUser)
{
if( (theUser) && (theUser->getIsSuspended()))
	{
	//Check if the suspend hadnt already expired
	if(::time( 0 ) - theUser->getSuspendExpires() < 0)
		{
		return true;
		}
	}
return false;
}

bool ccontrol::refreshSuspention()
{

int totalCount = 0;
usersIterator curUser = usersMap.begin();
ccUser* tempUser;
for(;curUser != usersMap.end();++curUser)
	{
	tempUser = curUser->second;
	if((tempUser->getIsSuspended()) && (tempUser->getSuspendExpires() < ::time(0)))
		{
		tempUser->setSuspendExpires(0);
		tempUser->setSuspendedBy("");
		tempUser->setIsSuspended(false);
		tempUser->setSuspendLevel(0);
		tempUser->setSuspendReason("");
		++totalCount;
		}
	}

static const char *DelMain = "update opers set isSuspended = 'n',Suspend_Expires = 0, Suspended_by = '',suspend_level = 0,suspend_reason='' where IsSuspended = 'y' And suspend_expires < now()::abstime::int4";

strstream DelQuery;
DelQuery	<< DelMain
		<< ends;
#ifdef LOG_SQL
elog	<< "ccontrol::RefreshSuspention> "
	<< DelQuery.str()
	<< endl; 
#endif
ExecStatusType status = SQLDb->Exec( DelQuery.str() ) ;
delete[] DelQuery.str() ;

if( PGRES_COMMAND_OK != status )
	{
	elog	<< "ccontrol::refreshSuspention> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;
		return false ;
	}
if(totalCount > 0)
	MsgChanLog("[Refresh Suspend] - %d expired\n",totalCount);		

return true;
}

bool ccontrol::refreshGlines()
{

if(!dbConnected)
	{
	return false;
	}

int totalFound = 0;
inRefresh = true;

for(glineIterator ptr = glineList.begin();ptr != glineList.end();) 
	{
	if(((*ptr)->getExpires() <= ::time(0)) 
	    && (((*ptr)->getHost().substr(0,1) != "#") || 
	    ((*ptr)->getExpires() != 0)))

		{
		//remove the gline from the database
		ccGline* tGline = *ptr;
		tGline->Delete();
		ptr = glineList.erase(ptr);
		delete tGline;
		++totalFound;
		}
	else
		ptr++;
	}

inRefresh = false;

//if(totalFound > 0)
//	MsgChanLog("[Refresh Glines] - %d expired\n",totalFound);
return true;

}

bool ccontrol::burstGlines()
{

ccGline *theGline = 0 ;
iServer* curServer;
unsigned int Expires = 0;
for(glineListType::iterator ptr = glineList.begin()
    ; ptr != glineList.end(); ++ptr)
	{
	theGline = *ptr;
	if((theGline->getExpires() == 0) && (theGline->getHost().substr(0,1) == "#"))
		{
		Expires =  gline::PERM_TIME;
		}
	else
		{
		Expires = theGline->getExpires() - ::time(0);
		}
	for(ccGline::burstIterator bPtr = theGline->getBurstBegin();
	    bPtr != theGline->getBurstEnd();++bPtr)
		{
		curServer = Network->findServer((**bPtr));
		if(curServer)
			{
			MyUplink->setGline(theGline->getAddedBy(),
				theGline->getHost(),
				theGline->getReason(),
			Expires,this,curServer->getName());
			}
		}
	theGline->clearBurst();
	}
return true;
}

bool ccontrol::loadGlines()
{
//static const char *Main = "SELECT * FROM glines where ExpiresAt > now()::abstime::int4";

if(!dbConnected)
	{
	return false;
	}

static const char *Main = "SELECT * FROM glines";

strstream theQuery;
theQuery	<< Main
		<< ends;

elog	<< "ccontrol::loadGlines> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK != status )
	{
	elog	<< "ccontrol::loadGlines> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;

	return false;
	}

ccGline *tempGline = NULL;

inRefresh = true;

for( int i = 0 ; i < SQLDb->Tuples() ; i++ )
	{
	tempGline =  new (std::nothrow) ccGline(SQLDb);
	assert( tempGline != NULL ) ;

	tempGline->setId(SQLDb->GetValue(i,0));
	tempGline->setHost(SQLDb->GetValue(i,1));
	tempGline->setAddedBy(SQLDb->GetValue(i,2)) ;
	tempGline->setAddedOn(static_cast< time_t >( atoi( SQLDb->GetValue(i,3) ) )) ;
	tempGline->setExpires(static_cast< time_t >( atoi( SQLDb->GetValue(i,4) ) )) ;
	tempGline->setReason(SQLDb->GetValue(i,5));
	addGline(tempGline);
	}
return true;	
} 

bool ccontrol::loadUsers()
{
 
if(!dbConnected)
        {   
        return false;
        }
   
strstream theQuery;
theQuery        << User::Query
                << ends;

elog    << "ccotrol::loadUsers> "
        << theQuery.str()
        << endl; 
 
ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if (PGRES_TUPLES_OK != status)
        {
        return false;
        }
ccUser* tempUser;
for(int i =0;i<SQLDb->Tuples();++i)
	{
	tempUser = new (std::nothrow) ccUser(SQLDb);
	tempUser->setID(atoi(SQLDb->GetValue(i, 0)));
        tempUser->setUserName(SQLDb->GetValue(i, 1));
        tempUser->setPassword(SQLDb->GetValue(i, 2));
	tempUser->setAccess(atol(SQLDb->GetValue(i, 3)));
	tempUser->setSAccess(atol(SQLDb->GetValue(i, 4)));
	tempUser->setFlags(atoi(SQLDb->GetValue(i, 5)));
	tempUser->setSuspendExpires(atoi(SQLDb->GetValue(i,6)));
	tempUser->setSuspendedBy(SQLDb->GetValue(i,7));
	tempUser->setServer(SQLDb->GetValue(i,8));
	tempUser->setIsSuspended(!strcasecmp(SQLDb->GetValue(i,9),"t"));
	if(!strcasecmp(SQLDb->GetValue(i,10),"t"))
		{
		tempUser->setType(operLevel::UHSLEVEL);
		}
	else if(!strcasecmp(SQLDb->GetValue(i,11),"t"))
		{
		tempUser->setType(operLevel::OPERLEVEL);
		}
	else if(!strcasecmp(SQLDb->GetValue(i,12),"t"))
		{
		tempUser->setType(operLevel::ADMINLEVEL);
		}
	else if(!strcasecmp(SQLDb->GetValue(i,13),"t"))
		{
		tempUser->setType(operLevel::SMTLEVEL);
		}
	else if(!strcasecmp(SQLDb->GetValue(i,14),"t"))
		{
		tempUser->setType(operLevel::CODERLEVEL);
		}
	tempUser->setLogs(!strcasecmp(SQLDb->GetValue(i,15),"t"));
	tempUser->setNeedOp(!strcasecmp(SQLDb->GetValue(i,16),"t"));
	tempUser->setEmail(SQLDb->GetValue(i,17));
	tempUser->setSuspendLevel(atoi(SQLDb->GetValue(i,18)));
	tempUser->setSuspendReason(SQLDb->GetValue(i,19));
	if(!strcasecmp(SQLDb->GetValue(i,20),"t"))
		{
		tempUser->setNotice(true);
		}
	else
		{
		tempUser->setNotice(false);
		}
	usersMap[tempUser->getUserName()]=tempUser;

	}
return true;
}

bool ccontrol::loadServers()
{
 
if(!dbConnected)
        {   
        return false;
        }
   
strstream theQuery;
theQuery        << server::Query
                << ends;

elog    << "ccotrol::loadServers> "
        << theQuery.str()
        << endl; 
 
ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if (PGRES_TUPLES_OK != status)
        {
        return false;
        }
ccServer* tempServer;
for(int i =0;i<SQLDb->Tuples();++i)
	{
	tempServer = new (std::nothrow) ccServer(SQLDb);
	assert(tempServer != NULL);
	tempServer->loadDataFromDB(i);
	serversMap[tempServer->getName()] = tempServer;
	}
return true;
}

	
bool ccontrol::loadMaxUsers()
{
 
if(!dbConnected)
        {   
        return false;
        }
   
strstream theQuery;
theQuery        << "Select * from misc where VarName = 'MaxUsers';"
                << ends;

elog    << "ccotrol::loadMaxUsers> "
        << theQuery.str()
        << endl; 
 
ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if (PGRES_TUPLES_OK != status)
        {
        elog << "Error on loading maxusers : " << SQLDb->ErrorMessage() << endl;
	return false;
        }
if(SQLDb->Tuples() == 0)
	{
	maxUsers = 0;
	dateMax = 0;
	strstream insertQ;
	insertQ << "Insert into misc (VarName,Value1,Value2) Values ('MaxUsers',0,0);"
		<< ends;

	status = SQLDb->Exec( insertQ.str() ) ;
	delete[] insertQ.str() ;

	if (PGRES_COMMAND_OK != status)
    		{
		return false;
	        }
	}
else
	{
	maxUsers = atoi(SQLDb->GetValue(0,1));
	dateMax = atoi(SQLDb->GetValue(0,2));
	}
return true;
}

bool ccontrol::loadVersions()
{
 
if(!dbConnected)
        {   
        return false;
        }
   
strstream theQuery;
 
ExecStatusType status = SQLDb->Exec( "Select * from misc where VarName = 'Version'") ;

if (PGRES_TUPLES_OK != status)
        {
        return false;
        }

for(int i =0;i<SQLDb->Tuples();++i)
	{
	VersionsList.push_back(SQLDb->GetValue(i,1));
	}

status = SQLDb->Exec( "Select * from misc where VarName = 'CheckVer'") ;

if (PGRES_TUPLES_OK != status)
        {
        return false;
        }

if(SQLDb->Tuples() == 0)
	{
	status = SQLDb->Exec( "insert into misc (VarName,Value1) VALUES ('CheckVer',0)") ;
	checkVer = false;
	if(PGRES_COMMAND_OK != status)
		{
		return false;
		}
	}
else
	{
	checkVer = atoi(SQLDb->GetValue(0,1));
	}
return true;
}

void ccontrol::wallopsAsServer(const char *Msg,...)
{
if( 0 == MyUplink )
	{
	return ;
	}

char buffer[ 1024 ] = { 0 } ;
va_list list;

va_start( list , Msg) ;
vsprintf( buffer, Msg , list ) ;
va_end( list ) ;

MyUplink->Wallops( buffer ) ;
}

int ccontrol::getExceptions( const string &Host )
{
int Exception = userMaxConnection;

for(exceptionIterator ptr = exception_begin();ptr != exception_end();ptr++)
	{
	if((*(*ptr) == Host) || !(match((*ptr)->getHost(),Host)))
		{
		if((*ptr)->getConnections() > Exception)
			{
			Exception = (*ptr)->getConnections();
			}
		} 
	}

return Exception;
}

bool ccontrol::listExceptions( iClient *theClient )
{

Notice(theClient,"-= Exceptions list - listing a total of %d exceptions =-"
	,exceptionList.size());
	
for(exceptionIterator ptr = exception_begin();ptr != exception_end();ptr++)
	Notice(theClient,"Host : %s  Connections : %d AddedBy : %s"
	       ,(*ptr)->getHost().c_str()
	       ,(*ptr)->getConnections()
	       ,(*ptr)->getAddedBy().c_str());

Notice(theClient,"-= End of exception list =-");

return true;
}

bool ccontrol::isException( const string & Host )
{
for(exceptionIterator ptr = exception_begin();ptr != exception_end();ptr++)
	{
	if(*(*ptr) == Host)
		return true;
	}
return false;
}


bool ccontrol::insertException( iClient *theClient , const string& Host , int Connections )
{
//Check if there is already an exception on that host

if(!dbConnected)
	{
	return false;
	}

if(isException(Host))
	{
	Notice(theClient,
		"There is already an exception for host %s, "
		"please use update",
		Host.c_str());		
	return true;
	}

//Create a new ccException structure 
ccException* tempException = new (std::nothrow) ccException(SQLDb);
assert(tempException != NULL);

tempException->setHost(removeSqlChars(Host));
tempException->setConnections(Connections);
tempException->setAddedBy(removeSqlChars(theClient->getNickUserHost()));
tempException->setAddedOn(::time(0));

//Update the database, and the internal list
if(!tempException->Insert())
	{
	return false;
	}

exceptionList.push_back(tempException);
return true;
}


bool ccontrol::delException( iClient *theClient , const string &Host )
{

if(!dbConnected)
	{
	return false;
	}

if(!isException(removeSqlChars(Host)))
	{
	Notice(theClient,"Cant find exception for host %s",Host.c_str());
	return true;
	}
ccException *tempException = NULL;

for(exceptionIterator ptr = exception_begin();ptr != exception_end();)
	{
	tempException = *ptr;
	if(*tempException == removeSqlChars(Host))
		{
		bool status = tempException->Delete();
		ptr = exceptionList.erase(ptr);
		delete tempException;
		if(!status)
			return false;
		}
	    
	else
		ptr++;
	}
return true;
}	

ccFloodData *ccontrol::findLogin( const string & Numeric )
{
for(ignoreIterator ptr = ignore_begin() ; ptr != ignore_end() ; ++ptr)
	{
	if((*ptr)->getNumeric() == Numeric)
		{
		return *ptr;
		}
	}
return NULL;
}

void ccontrol::removeLogin( ccFloodData *tempLogin )
{
for(ignoreIterator ptr = ignore_begin() ; ptr != ignore_end() ;)
	{
	if((*ptr) == tempLogin)
		{
		ptr = ignoreList.erase(ptr);
		}
	else
		ptr++;
	}
}

void ccontrol::addLogin( iClient* tClient)
{
ccFloodData *LogInfo = static_cast<ccUserData*>( tClient->getCustomData(this))->getFlood();

LogInfo->add_Login();
if(LogInfo->getLogins() > 5)
	{
	ignoreUser(LogInfo);
	}
}	

int ccontrol::removeIgnore( const string &Host )
{

ccFloodData *tempLogin = 0;
int retMe = IGNORE_NOT_FOUND;

for(ignoreIterator ptr = ignore_begin();ptr!=ignore_end();)
	{
	tempLogin = *ptr;
	if(tempLogin->getIgnoredHost() == Host)
		{
		strstream s;
		s	<< getCharYYXXX() 
			<< " SILENCE " 
			<< tempLogin->getNumeric() 
			<< " -" 
			<< tempLogin->getIgnoredHost()
			<< ends; 
		Write( s );
		delete[] s.str();
		tempLogin->resetIgnore();
		tempLogin->resetLogins();
		ptr = ignoreList.erase(ptr);
		if(tempLogin->getNumeric() == "0")
			{
			delete tempLogin;
			}
		retMe = IGNORE_REMOVED;
		}
	else
		ptr++;
	}
return retMe;
}	

int ccontrol::removeIgnore( iClient *theClient )
{
string Host = string( "*!*" )
		+ theClient->getUserName() 
		+ string( "@" )
		+ theClient->getInsecureHost();
int retMe = removeIgnore(Host);
return retMe;
}

void ccontrol::ignoreUser( ccFloodData* Flood )
{

const iClient* theClient = Network->findClient(Flood->getNumeric());
Notice(theClient,"I dont think I like you anymore , consider yourself ignored");
MsgChanLog("Added %s to my ignore list\n",theClient->getNickUserHost().c_str());

string silenceMask = string( "*!*" )
	+ theClient->getUserName()
	+ "@"
	+ theClient->getInsecureHost();

strstream s;
s	<< getCharYYXXX() 
	<< " SILENCE " 
	<< theClient->getCharYYXXX() 
	<< " " 
	<< silenceMask
	<< ends; 

Write( s );
delete[] s.str();
Flood->setIgnoreExpires(::time(0)+ flood::IGNORE_TIME);
Flood->setIgnoredHost(silenceMask);

ignoreList.push_back(Flood);
}

bool ccontrol::listIgnores( iClient *theClient )
{
Notice(theClient,"-= Listing Ignore List =-");			
ccFloodData *tempLogin;
for(ignoreIterator ptr = ignore_begin();ptr!=ignore_end();ptr++)
	{
	tempLogin = *ptr;
	if(tempLogin->getIgnoreExpires() > ::time(0))
		{
		Notice(theClient,"Host : %s Expires At %s[%d]",
		tempLogin->getIgnoredHost().c_str(),
		convertToAscTime(tempLogin->getIgnoreExpires()),
			tempLogin->getIgnoreExpires());
		}
	}
Notice(theClient,"-= End Of Ignore List =-");			
return true ;
}

bool ccontrol::refreshIgnores()
{
ccFloodData *tempLogin;
unsigned int TotalFound =0;
for(ignoreIterator ptr = ignore_begin();ptr!=ignore_end();)
	{
	tempLogin = *ptr;
	if((tempLogin) &&(tempLogin->getIgnoreExpires() <= ::time(0)))
		{
		tempLogin->setIgnoreExpires(0);
		strstream s;
		s	<< getCharYYXXX() 
			<< " SILENCE " 
			<< tempLogin->getNumeric() 
			<< " -" 
			<< tempLogin->getIgnoredHost()
			<< ends; 

		Write( s );
		delete[] s.str();
		tempLogin->setIgnoredHost("");
		if(tempLogin->getNumeric() == "0")
			{
			delete tempLogin;
			}
		ptr = ignoreList.erase(ptr);
		++TotalFound;
		}
	else
		ptr++;
	}
if(TotalFound > 0)
	MsgChanLog("[Refresh Ignores] - %d expired\n",TotalFound);

return true;

}
bool ccontrol::loadExceptions()
{
static const char *Main = "SELECT * FROM Exceptions";

if(!dbConnected)
	{
	return false;
	}

strstream theQuery;
theQuery	<< Main
		<< ends;

elog	<< "ccontrol::loadExceptions> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK != status )
	{
	elog	<< "ccontrol::loadExceptions> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	
	return false;
	}

ccException *tempException = NULL;

for( int i = 0 ; i < SQLDb->Tuples() ; i++ )
	{
	tempException =  new (std::nothrow) ccException(SQLDb);
	assert( tempException != 0 ) ;

	tempException->setHost(SQLDb->GetValue(i,0));
	tempException->setConnections(atoi(SQLDb->GetValue(i,1)));
	tempException->setAddedBy(SQLDb->GetValue(i,2)) ;
	tempException->setAddedOn(static_cast< time_t >( atoi( SQLDb->GetValue(i,3) ) )) ;
	exceptionList.push_back(tempException);
	}
return true;	
} 

void ccontrol::listGlines( iClient *theClient, string Mask )
{

ccGline* tempGline;
Notice(theClient,"-= Gline List =-");
for(glineIterator ptr = gline_begin();ptr != gline_end();++ptr)
	{
	tempGline =*ptr;
	if((tempGline ->getExpires() > ::time(0)) 
	    && (!match(Mask,tempGline->getHost())))
		{
		Notice(theClient,"Host : %s , Expires At : %s[%d] , AddedBy %s"
			,tempGline->getHost().c_str()
			,convertToAscTime(tempGline->getExpires())
			,tempGline->getExpires()
			,tempGline->getAddedBy().c_str());
		}
	}
Notice(theClient,"-= End Of Gline List =-");

}
			
void ccontrol::listSuspended( iClient * )
{

}

	
void ccontrol::listServers( iClient * theClient)
{
ccServer* tmpServer;
for(serversConstIterator ptr = serversMap_begin();ptr != serversMap_end();++ptr)
	{
	tmpServer = ptr->second;
	if(tmpServer->getNetServer())
		{
		Notice(theClient,"%s (%d) - Name : %s",tmpServer->getLastNumeric().c_str()
		,base64toint(tmpServer->getLastNumeric().c_str()),
		tmpServer->getName().c_str());
		}
	else
		{
		Notice(theClient,"%s (%d) - Name : %s *MISSING*",tmpServer->getLastNumeric().c_str()
		,base64toint(tmpServer->getLastNumeric().c_str()),
		tmpServer->getName().c_str());
		}
	Notice(theClient,"Version : %s" ,tmpServer->getVersion().c_str());
	}
	 
}


void ccontrol::loadCommands()
{

static const char *Main = "SELECT * FROM Commands";

if(!dbConnected)
	{
	return;
	}

strstream theQuery;
theQuery	<< Main
		<< ends;

elog	<< "ccontrol::loadCommands> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK != status )
	{
	elog	<< "ccontrol::loadCommands> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;

	return;
	}

Command* NewCom;

for( int i = 0 ; i < SQLDb->Tuples() ; i++ )
	{
	NewCom = findRealCommand(SQLDb->GetValue(i,0));
	if(!NewCom)
		{
		elog	<< "Can't find handler for command "
			<< SQLDb->GetValue(i,0)
			<< endl;	
		}
	else
		{
		NewCom->setName(SQLDb->GetValue(i,1));
		if(!strcasecmp(SQLDb->GetValue(i,3),"f"))
			NewCom->Enable();
		else
			NewCom->Disable();
		NewCom->setNeedOp(!strcasecmp(SQLDb->GetValue(i,4),"t") ? true : false);			
		NewCom->setNoLog(!strcasecmp(SQLDb->GetValue(i,5),"t") ? true : false);			

		}
	}

}

	
bool ccontrol::updateCommand ( Command* Comm)
{
static const char *Main = "UPDATE Commands set name = '";

if(!dbConnected)
	{
	return false;
	}

strstream theQuery;
theQuery	<< Main
		<< Comm->getName()
		<< "', isDisabled = "
		<< (Comm->getIsDisabled() ? "'t'" : "'n'")
		<< ", NeedOp = "
		<< (Comm->getNeedOp() ? "'t'" : "'n'")
		<< ", NoLog = "
		<< (Comm->getNoLog() ? "'t'" : "'n'")
		<< ", MinLevel = "
		<< Comm->getMinLevel() 
		<< " Where lower(RealName) = '"
		<< string_lower(Comm->getRealName())
		<< "'"
		<< ends;

elog	<< "ccontrol::updateCommands> "
	<< theQuery.str()
	<< endl; 

ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_COMMAND_OK != status )
	{
	elog	<< "ccontrol::updateCommands> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;

	return false;
	}

RegisterCommand(Comm);
return true;

}
	
Command* ccontrol::findRealCommand( const string& commName)
{
for( commandMapType::iterator ptr = commandMap.begin() ;
	ptr != commandMap.end() ; ++ptr )
	if(!strcasecmp(ptr->second->getRealName(),commName))
		return ptr->second;
return NULL;
}

Command* ccontrol::findCommandInMem( const string& commName)
{
for( commandMapType::iterator ptr = commandMap.begin() ;
	ptr != commandMap.end() ; ++ptr )
	if(!strcasecmp(ptr->second->getName(),commName))
		return ptr->second;
return NULL;
}

		
bool ccontrol::UpdateCommandFromDb ( Command* Comm )
{
static const char *Main = "SELECT * From Commands where lower(RealName) = '";

if(!dbConnected)
	{
	return false;
	}

strstream theQuery;
theQuery	<< Main
		<< string_lower(Comm->getRealName())
		<< "'" << ends;


ExecStatusType status = SQLDb->Exec( theQuery.str() ) ;
delete[] theQuery.str() ;

if( PGRES_TUPLES_OK != status )
	{
	elog	<< "ccontrol::LoadCommand> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;

	return false;
	}
if(SQLDb->Tuples() == 0)
	return false;
Comm->setName(SQLDb->GetValue(0,1));
if(!strcasecmp(SQLDb->GetValue(0,3),"t"))
	Comm->Disable();
else
	Comm->Enable();
Comm->setNeedOp((!strcasecmp(SQLDb->GetValue(0,4),"t")) ? true : false);
Comm->setNoLog((!strcasecmp(SQLDb->GetValue(0,5),"t")) ? true : false);
Comm->setMinLevel(atoi(SQLDb->GetValue(0,6)));

return true;

}

const string ccontrol::expandDbServer(const string& Name)
{

serversMapType::iterator serverIt;
for(serverIt = serversMap.begin();serverIt != serversMap.end();++serverIt)
	{
	if(!match(Name,serverIt->first))
		return serverIt->first;
	}
return "";
/*ccServer* tmpServer = getServer(Name);
if(tmpServer)
	return tmpServer->getName();
return "";*/

}

const string ccontrol::removeSqlChars(const string& Msg)
{
string NewString;

for(string::const_iterator ptr = Msg.begin(); ptr != Msg.end() ; ++ptr)
	{
	if(*ptr == ';')
		{
		NewString += ' ';
		}
	else if(*ptr == '\'')
		{
		NewString += "\\\047";
		}
	else if(*ptr == '\\')
		{
		NewString += "\\\134";
		}
	else
		{
		NewString += *ptr;
		}
	}
return NewString;

}

void ccontrol::GatesCheck()
{
gateIterator ptr;
ccGate* tmpGate;
for(ptr = gatesCheckingQueue.begin();ptr!=gatesCheckingQueue.end();)
	{
	tmpGate = *ptr;
	if(tmpGate->getStatus() == ccGate::statDone)
		{
		if(tmpGate->getFound())
			{
			MsgChanLog("Found socks proxy in %s\n",tmpGate->getHost().c_str());
			}			
		ptr = gatesCheckingQueue.erase(ptr);
		delete tmpGate;
		}
	else
		{
		++ptr;
		}
	}

int LeftThreads = maxThreads - gatesCheckingQueue.size();
//MsgChanLog("Left threads is %d",LeftThreads);
ptr = gatesWaitingQueue.begin();
int er;
pthread_t tId; 
for(;(!(gatesWaitingQueue.empty()) && (LeftThreads > 0));--LeftThreads)
	{
	tmpGate = gatesWaitingQueue.front();
	//gatesWaitingQueue.pop_front();
	gatesWaitingQueue.erase(gatesWaitingQueue.begin());
	//MsgChanLog("After %d\n",gatesWaitingQueue.size());
	gatesCheckingQueue.push_back(tmpGate);
	if((er = pthread_create(&tId,NULL,initGate,(void *)tmpGate)) < 0)
		{
		MsgChanLog("Error(%d) while creating thread\n",errno);
		gatesCheckingQueue.pop_front();
		delete tmpGate;
		}
	else
		{
		tmpGate->setThreadId(tId);
		}
	}

}	

void ccontrol::checkDbConnection()
{

if(SQLDb->Status() == CONNECTION_BAD) //Check if the connection had died
	{
	delete(SQLDb);
	dbConnected = false;
	updateSqldb(NULL);
	MsgChanLog("PANIC! - The Connection With The Db Was Lost\n");
	MsgChanLog("Attempting to reconnect, Attempt %d out of %d\n"
		    ,connectCount+1,connectRetry+1);
	string Query = "host=" + sqlHost + " dbname=" + sqlDb + " port=" + sqlPort;
	if (strcasecmp(sqlUser,"''"))
		{
		Query += (" user=" + sqlUser);
		}

	if (strcasecmp(sqlPass,"''"))
		{
		Query += (" password=" + sqlPass);
		}
	SQLDb = new (std::nothrow) cmDatabase(Query.c_str());
	assert(SQLDb != NULL);
	
	if(SQLDb->ConnectionBad())
		{
		++connectCount;
		if(connectCount > connectRetry)
			{
			MsgChanLog("Cant connect to the database, quiting\n");
			::exit(1);
			}
		else
			{
			MsgChanLog("Attempt was failed\n");
			}
		}
	else
		{
		dbConnected = true;
		MsgChanLog("The PANIC is over, db connection restored\n");
		updateSqldb(SQLDb);
		connectCount = 0;
		}
	}
	
	
}

void ccontrol::updateSqldb(PgDatabase* _SQLDb)
{

for(glineIterator ptr = glineList.begin();ptr != glineList.end();++ptr) 
	{
	(*ptr)->setSqldb(_SQLDb);
	}

for(exceptionIterator ptr = exception_begin();ptr != exception_end();++ptr)
	{
	(*ptr)->setSqldb(_SQLDb);
	}

for(usersIterator ptr = usersMap.begin();ptr != usersMap.end();++ptr)
	{
	ptr->second->setSqldb(_SQLDb);
	}

for(serversIterator ptr = serversMap.begin();ptr != serversMap.end();++ptr)
	{
	ptr->second->setSqldb(_SQLDb);
	}

}

void ccontrol::showStatus(iClient* tmpClient)
{
int uptime = ::time(0) - getUplink()->getStartTime();
int days;
int hours;
int mins;
int secs;
unsigned long  totalRecv = getUplink()->getTotalReceived() / uptime;
unsigned long totalSent = getUplink()->getTotalSent() / uptime;

days = uptime/(24*3600);
uptime %= 24*3600;
hours = uptime / 3600;
uptime %= 3600;
mins = uptime / 60;
uptime %= 60;
secs = uptime;
Notice(tmpClient,"Uptime : %dD %dH %dM %dS",days,hours,mins,secs);
if(checkClones)
	{
	Notice(tmpClient,"Monitoring %d diffrent clones hosts\n",clientsIpMap.size());
	Notice(tmpClient,"and %d diffrent virtual clones hosts\n",virtualClientsMap.size());
	}	
if(checkGates)
	Notice(tmpClient,"Monitoring %d threads out of %d, and there are %d in the waiting queue",gatesCheckingQueue.size()
		,maxThreads,gatesWaitingQueue.size());
Notice(tmpClient,"Allocated Structures:");
Notice(tmpClient,"ccGate:  %d",ccGate::numAllocated);
Notice(tmpClient,"ccServer: %d",ccServer::numAllocated);
Notice(tmpClient,"ccGline: %d",ccGline::numAllocated);
Notice(tmpClient,"ccException: %d",ccException::numAllocated);
Notice(tmpClient,"ccUser: %d",ccUser::numAllocated);
Notice(tmpClient,"Total of %d users in the map",usersMap.size()); 

if(totalRecv > 1024*1024)
	{
	Notice(tmpClient,"Received a total of %ld (average of %ld MB/Sec)",
		getUplink()->getTotalReceived(),double(totalRecv)/1024*1024);
	}
else
	{
	Notice(tmpClient,"Received a total of %ld bytes (average of %ld Bytes/Sec)",
		getUplink()->getTotalReceived(),totalRecv);	
	}
	
if(totalSent > 1024*1024)
	{
	Notice(tmpClient,"Sent a total of %ld bytes (average of %ld MB/Sec)",
		getUplink()->getTotalSent(),double(totalSent)/1024*1024);
	}
else
	{
	Notice(tmpClient,"Sent a total of %ld bytes (average of %ld Bytes/Sec)",
		getUplink()->getTotalSent(),totalSent);	
	}

//Notice(tmpClient,"Received a total of %ld bytes",getUplink()->getTotalReceived());
}

unsigned int ccontrol::checkPassword(string NewPass , ccUser* tmpUser)
{
if(NewPass.size() < password::MIN_SIZE)
	{
	return password::TOO_SHORT;
	}
if(!strcasecmp(NewPass,tmpUser->getUserName()))
	{
	return password::LIKE_UNAME;
	}
return password::PASS_OK;
}

ccServer* ccontrol::getServer(const string& Name)
{

serversMapType::iterator serverIt;
serverIt = serversMap.find(Name);
if(serverIt != serversMap.end())
	{
	return serverIt->second;
	}
return NULL;

/*ccServer* tempServer = serversMap[Name];
if(tempServer)
	return tempServer;
serversMap.erase(serversMap.find(Name));
return NULL;*/

}


void ccontrol::addServer(ccServer* tempServer)
{
if(!serversMap[tempServer->getName()])
	{
	serversMap[tempServer->getName()] = tempServer;
	}
}

void ccontrol::remServer(ccServer* tempServer)
{
serversMap.erase(serversMap.find(tempServer->getName()));
}

#ifdef LOGTOHD
void ccontrol::initLogs()
{
//TODO: Get this from the conf file
LogFileName = "CommandsLog.Log";
//TODO: Get this from the conf file
LogsToSave = 100;
NumOfLogs = 0;
LogFile.open(LogFileName.c_str(),ios::in|ios::out);

if(LogFile.bad())
	{//There was a problem in opening the log file
	elog << "Error while initilizing the logs file!\n";
	return ;
	}
//LogFile.setbuf(NULL,0);
LogFile.close();
}

void ccontrol::addLog(ccLog* newLog)
{
ccLog* oldLog;
while(LogList.size() >= LogsToSave)
	{
	oldLog = LogList.back();
	LogList.pop_back();
	delete oldLog;
	}
LogList.push_front(newLog);
}

/*
 ccontrol::showLogs - sends the lastcom log to a client
 loading the logs from the file is done using the 
 lazy evaluation tactic, meaning it only loads the data from
 the hardisk if it has to , thous saving time

*/

void ccontrol::showLogs(iClient* theClient, unsigned int Amount)
{

if(Amount > LogsToSave)
	{
	Notice(theClient,"Sorry, but you can't view more than the last %d commands"
		,LogsToSave);
	Amount = LogsToSave;
	}
	

if((LogList.size() < Amount) 
	&& ((NumOfLogs > LogList.size()) || (NumOfLogs == 0)))
	{
	if((!LogFile.eof() && LogFile.bad()) || !(LogFile.is_open()))
		{
		LogFile.close();
		LogFile.open(LogFileName.c_str(),ios::in|ios::out);
		//LogFile.setbuf(NULL,0);
		if(LogFile.bad())
			{
			Notice(theClient,"Error while reading the lastcom report");
			MsgChanLog("Error while reading from the lastcom file!\n");
			return;
			}
		}
	//Clean the list first
	for(ccLogIterator ptr= LogList.begin(); ptr != LogList.end();)
		{
		delete *ptr;
		ptr = LogList.erase(ptr);
		}
	/*
	    since every record has its own size, there is no way of knowing
	    where in the file the last X records are, so we start reading
	    from the begining of the file, this saves hd space but may 
	    cost some time.
	    
	    this is done only once if at all per restart, so its
	    not a big deal *g*
	*/
	ccLog* tmpLog;
	LogFile.seekg(0,ios::beg);
	NumOfLogs = 0;
	while(!LogFile.eof())
		{
		tmpLog = new (std::nothrow) ccLog();
		if(!tmpLog->Load(LogFile))
			{
			if(!LogFile.eof())
				{
				Notice(theClient,"Error while reading the lastcom report");
				return;
				}
			}
		else
			{
			++NumOfLogs;
			addLog(tmpLog); 
			}
		}
	LogFile.close();
//	LogFile.open(LogFileName.c_str(),ios::in|ios::out);
//	LogFile.setbuf(NULL,0);
	delete tmpLog;
	}
//At this point, we should have the log list full of the last LogsToSave
//commands, and we need to show only Amount of them
unsigned int Left = LogList.size();
if(Left == 0)
	return;
ccLogIterator curLog = LogList.end();
while(Left > Amount)
	{
	--curLog;
	Left--;
	}
if(curLog == LogList.end())
	{
	--curLog;
	}
while(Left > 0)
	{
	Notice(theClient,"[%s] - [(%s) - %s] - %s"
		,convertToAscTime((*curLog)->Time)
		,(*curLog)->User.c_str()
		,(*curLog)->Host.c_str()
		,(*curLog)->Desc.c_str());
	--curLog;
	Left--;
	}

}

#endif

int ccontrol::OnSignal(int sig)
{ 
if(sig == SIGUSR1)
	saveServersInfo();
else
if(sig == SIGUSR2)
	saveChannelsInfo();

return xClient::OnSignal(sig);
}

void ccontrol::saveServersInfo()
{
ofstream servFile("ServerList.txt",ios::out);
if(!servFile)
	{
	elog << "Error while opening server list file!\n";
	return;
	}
gnuworld::xNetwork::const_serverIterator sIterator = Network->server_begin();
iServer* curServer;
for(;sIterator != Network->server_end();++sIterator)
	{
	curServer = *sIterator;
	if(!curServer)
		continue;
	servFile << curServer->getName().c_str()
		 << " " << curServer->getConnectTime()
		 << " " << Network->countClients(curServer)
		 << endl;
	}
servFile.close();
}

void ccontrol::saveChannelsInfo()
{
ofstream chanFile("ChannelsList.txt",ios::out);
if(!chanFile)
	{
	elog << "Error while opening server list file!\n";
	return;
	}
gnuworld::xNetwork::constChannelIterator cIterator = Network->channels_begin();
Channel* curChannel;
for(;cIterator != Network->channels_end();++cIterator)
	{
	curChannel = cIterator->second;
	if(!curChannel)
		continue;
	chanFile << curChannel->getName()
		 << " " << curChannel->getCreationTime()
		 << " " << curChannel->getModeString()
		 << " " << curChannel->size()
#ifdef TOPIC_TRACK
		 << " " << curChannel->getTopic()
#endif
		 << endl;
	}
chanFile.close();
}

void ccontrol::checkMaxUsers()
{
if(maxUsers < curUsers)
	{
	maxUsers = curUsers;
	dateMax = ::time(0);
	
	static const char *UPMain = "update Misc set Value1 = ";

	strstream DelQuery;
	DelQuery	<< UPMain
			<< maxUsers
			<< ", Value2 = " 
			<< dateMax
			<< " Where VarName = 'MaxUsers'"
			<< ends;
//#ifdef LOG_SQL
	elog	<< "ccontrol::checkMaxUsers> "
		<< DelQuery.str()
		<< endl; 
//#endif
	ExecStatusType status = SQLDb->Exec( DelQuery.str() ) ;
	delete[] DelQuery.str() ;

	if( PGRES_COMMAND_OK != status )
		{
		elog	<< "ccontrol::checkMaxUsers> SQL Failure: "
			<< SQLDb->ErrorMessage()
			<< endl ;
		}

	}
}

bool ccontrol::addVersion(const string& newVer)
{
static const char* verChar = "Insert into misc (VarName,Value5) Values ('Version','";
strstream VerQuery;
VerQuery	<< verChar
		<< newVer
		<< "')" 
		<< ends;
#ifdef LOG_SQL
elog		<< "ccontrol::addVersion> "
		<< VerQuery.str()
		<< endl; 
#endif
ExecStatusType status = SQLDb->Exec( VerQuery.str() ) ;
delete[] VerQuery.str() ;
if( PGRES_COMMAND_OK != status )
	{
	elog	<< "ccontrol::addVersion> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	}

return true;
}

bool ccontrol::remVersion(const string& oldVer)
{
versionsIterator ptr = VersionsList.begin();
for(;ptr != VersionsList.end();)
	{
	if(!strcasecmp(oldVer,*ptr))
		{ //Found the version in the list
		ptr = VersionsList.erase(ptr);
		}
	else
		{
		++ptr;
		}
	}

string delS = "delete from misc where VarName = 'Version' and lower(Value5) = '" + string_lower(oldVer) + "'";
return (PGRES_COMMAND_OK == SQLDb->Exec(delS.c_str()));
return true;
}

bool ccontrol::isValidVersion(const string& checkVer)
{
versionsIterator ptr = VersionsList.begin();
for(;ptr != VersionsList.end();++ptr)
	{
	if(!strcasecmp(checkVer,*ptr))
		{ //Found the version in the list
		return true;
		}
	}
return false;
}

void ccontrol::listVersions(iClient* theClient)
{
}

bool ccontrol::updateCheckVer(const bool newVal)
{
checkVer = newVal;
strstream ups;
ups 	<< "Update misc set Value1 = "
	<< (newVal ? 1 : 0)
        << " where VarName = 'CheckVer'"
	<< ends;

ExecStatusType status = SQLDb->Exec( ups.str() ) ;
delete[] ups.str() ;
if( PGRES_COMMAND_OK != status )
	{
	elog	<< "ccontrol::updateCheckVer> SQL Failure: "
		<< SQLDb->ErrorMessage()
		<< endl ;
	return false;
	}
return true;
}

void *initGate(void* arg)
{

pthread_detach(pthread_self()); //First we must de-attach 
ccGate * tmpGate = (ccGate*) arg;


int sockFd,conRet;
fd_set ReadSet,WriteSet;
bool error = false;
int tError = 0;
char Buf[512] = {0};
struct sockHead{
    char Version; //Socks version number
    char Command; //Connect command = 1
    short int Port; //Destination port
    char destIp[4]; //Destination ip 
    char userid[5]; //Userid
    };
struct tval{
    long int secs;
    long int milsecs;
};
timeval Timeout;
Timeout.tv_sec = 3;
Timeout.tv_usec = 0;

sockHead sockHeader;
sockHeader.Version = 4;
sockHeader.Command = 1;
sockHeader.Port = htons(6667);
sockHeader.destIp[0] = 127;
sockHeader.destIp[1] = 0;
sockHeader.destIp[2] = 0;
sockHeader.destIp[3] = 1;
strcpy(sockHeader.userid,"blah");
unsigned int Port = tmpGate->getPort();
string Host = tmpGate->getHost();
struct sockaddr_in MyAddr;
MyAddr.sin_family = AF_INET;
MyAddr.sin_port = htons(Port);
MyAddr.sin_addr.s_addr = inet_addr(Host.c_str());
sockFd = socket(AF_INET,SOCK_STREAM,0);
int sockFlags = fcntl(sockFd,F_GETFL,0);
fcntl(sockFd,F_SETFL,sockFlags | O_NONBLOCK);
if((conRet = ::connect(sockFd,(struct sockaddr*) &MyAddr,sizeof(MyAddr)))< 0)
        if(errno != EINPROGRESS)
                error = true;
        else if(conRet != 0)  
                {
		FD_ZERO(&ReadSet);
                FD_SET(sockFd,&ReadSet);
                WriteSet = ReadSet;
		if((conRet = select(sockFd+1,&ReadSet,&WriteSet,NULL,&Timeout)) == 0)
			{
			error = true;
			}
		else if(FD_ISSET(sockFd,&ReadSet) || FD_ISSET(sockFd,&WriteSet))
			{
			socklen_t len = sizeof(tError);
			if(getsockopt(sockFd,SOL_SOCKET,SO_ERROR,&tError,&len) < 0)
				error = true;
			}
		else
			error = true;
		}
if((!error) && !(tError))
	{
	fcntl(sockFd,F_SETFL,sockFlags);
	tmpGate->setStatus(ccGate::statConnected);
	int Res = 0;
	if(tmpGate->getPort() == 1080)
		{
		int trys = 10;
		do
			{
			errno =0;
			Res = ::send(sockFd,(void *) &sockHeader,sizeof(sockHeader)-1,0);
			}while((--trys) && (errno == EINTR));
		}
	if(Res >= 0)
		{
		FD_ZERO(&ReadSet);
		FD_SET(sockFd,&ReadSet);
		
		Res = ::select(sockFd+1,&ReadSet,NULL,NULL,&Timeout);
		if(Res >= 0)
			{
			if(FD_ISSET(sockFd,&ReadSet))
				{
				Res = ::recv(sockFd,Buf,sizeof(Buf),0);
				if(Res >= 0)
					{
					if(tmpGate->getPort() == 1080)
						{
						if(Buf[1] == 90) //Found a socks! 
							tmpGate->setFound(true);
						}
					else if(tmpGate->getPort() == 23)
						{
						if(strstr(Buf,"wingate>") != NULL)
							tmpGate->setFound(true);
						}
						
		    			}
				}
			}
		}
	}
close(sockFd);		
tmpGate->setStatus(ccGate::statDone);
pthread_exit(NULL);
return NULL;
}


}

} // namespace gnuworld
